#include <config.h>
#define _GNU_SOURCE
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
static const char input_variables_definition_arr[] =
"## Definitions of the input variables\n"
"##inputvar: #name of the variable as declared in the code\n"
"## DESCRIPTION: Extended description of the variable to be used for documentation\n"
"## COMMENT: This is the description of the variable as will appear in the logfile\n"
"## RANGE: [from,to] #always put two numbers (also .inf can be put)\n"
"## EXCLUSIVE: #here follows a mapping of allowed values (either RANGE or EXCLUSIVE)\n"
"##  Value1:  comments of value1\n"
"##  Value2:  comment of value2\n"
"## CONDITION: #here the conditions for which the variable makes sense are written\n"
"##   MASTER_KEY: foo #this means that inputvar makes sense only if foo is specified\n"
"##   WHEN: #provide a list of allowed values of foo for which inputvar is meaningful\n"
"##     - fooval1\n"
"##     - fooval2\n"
"## PROFILE_FROM: foo #inputvar inherit the profile from the one specified by foo variable\n"
"###then the profiles follows, which gives to the variables the allowed name\n"
"## default: value of the default, written as a string\n"
"## profile1: value1 # if the user specifies inputvar: profile1 then inputvars will be value1\n"
"## profile2: value2\n"
"##\n"
"## DO NOT USE the character ':' and "" in the COMMENT or DESCRIPTION\n"
"############################################\n"
"# Density Functional Theory parameters\n"
" dft:\n"
"   DESCRIPTION: |\n"
"     Defines all the parameters required for a single wavefunction calculation.\n"
"   hgrids:\n"
"     COMMENT: Grid spacing in the three directions (bohr)\n"
"     DESCRIPTION: |\n"
"      Grid spacing in three directions (Bohr units) of the coarse mesh as [0.5, 0.45, 0.4].\n"
"      A scalar can also be given as 0.45.\n"
"      The grid spacing of the Cartesian grid, in Bohr.\n"
"      As described in the wavelet formalism section, the nodes of this grid serve as the centers for the scaling\n"
"      function/wavelet basis.\n"
"      Values are in most cases between 0.3 and 0.6 bohr.\n"
"     RANGE: [0., 4.]\n"
"     default: [0.45, 0.45, 0.45]\n"
"     #fast: [0.55, 0.55, 0.55]\n"
"     #accurate: [0.30, 0.30, 0.30]\n"
"   ngrids:\n"
"     COMMENT: Number of grid spacing division in each direction\n"
"     DESCRIPTION: |\n"
"       Number of grid spacing division in each direction. Values are\n"
"       relevant only for periodic directions. This is convenient to keep\n"
"       the constant the number of wavelets when changing the super-cell dimensions.\n"
"       These values override hgrid ones.\n"
"     RANGE: [0, 1000]\n"
"     default: [0, 0, 0]\n"
"     auto: [0, 0, 0]\n"
"   rmult:\n"
"     COMMENT: c(f)rmult*radii_cf(:,1(2))=coarse(fine) atom-based radius\n"
"     DESCRIPTION: Coarse and fine multiplier as [4.0, 8.0].\n"
"                 These numbers determine the extension of the coarse and fine region respectively.\n"
"     RANGE: [0., 20.]\n"
"     default: [5., 8.]\n"
"   ixc:\n"
"     COMMENT: Exchange-correlation parameter (LDA=1,PBE=11)\n"
"     DESCRIPTION: |\n"
"       Determine the exchange-correlation functional.\n"
"       Integer specifying which exchange correlation functional will be used.\n"
"       The Abinit conventions are used and detailed information can be found on the Abinit Web page.\n"
"       Here is only a short summary of some widely used functionals\n"
"\n"
"       - Pade LDA from Abinit XC library;\n"
"       - PBE from Abinit XC library;\n"
"       - Pade LDA from libXC;\n"
"       - PBE from libXC;\n"
"       - PBEsol from libXC;\n"
"       - PBE0 Hybrid functional (local part from libXC).\n"
"\n"
"       A positive integer refers to a functional from ABINIT library and a negative one from libXC library.\n"
"       In this case, you concatenate the codes for the exchange part and for the correlation part (see the table of codes).\n"
"     default: 1\n"
"     #Here follow a number of possibilities for the different XC functionals\n"
"     LDA (ABINIT): 1\n"
"     PBE (ABINIT): 11\n"
"     LDA: -20\n"
"     PBE: -101130\n"
"     PBE0: -406\n"
"     B97-D: -170 #to be verified\n"
"     B3LYP: -402\n"
"     HARTREE: 0 #No exchange-correlation potential\n"
"     HF: 100 #Hartree-Fock\n"
"     NONE: 1000 #No Hartree and No XC (useful for H atom)\n"
"   qcharge:\n"
"     COMMENT: Charge of the system. Can be integer or real.\n"
"     DESCRIPTION: |\n"
"      Total charge of the system.\n"
"      A positive qcharge means that electrons are taken away and a negative one means that extra electrons are added.\n"
"     RANGE: [-500., 500.]\n"
"     default: 0\n"
"   elecfield:\n"
"     COMMENT: Electric field (Ex,Ey,Ez)\n"
"     DESCRIPTION: |\n"
"      Uniform electric field Ex, Ey, Ez in units of Ha/Bohr.\n"
"      The electric field can not have components in periodic directions, e.g. Ex = Ez = 0 in case surface BC is used.\n"
"     default: [0., 0., 0.]\n"
"   nspin:\n"
"     COMMENT: Spin polarization treatment\n"
"     DESCRIPTION: |\n"
"            Spin polarization treatment;\n"
"            1 no spin (as LDA),\n"
"            2 spin-polarisation (LSD or collinear) and\n"
"            4 non-collinear i.e. two components per electron.\n"
"     EXCLUSIVE:\n"
"       1: No spin\n"
"       2: Collinear\n"
"       4: Non-collinear\n"
"     default: 1\n"
"   mpol:\n"
"     COMMENT: Total magnetic moment\n"
"     DESCRIPTION: |\n"
"       Total magnetic moment which has to be the same parity as the total number of electrons (nelec_up-nelec_down=mpol).\n"
"       You should also define the keyword 'IGSpin' or add a spin per atom to specify the polarisation for the input guess.\n"
"     default: 0\n"
"   gnrm_cv:\n"
"     COMMENT: Convergence criterion gradient\n"
"     DESCRIPTION: |\n"
"       Convergence criterion for the wavefunction optimization (norm of the gradient).\n"
"       Reasonable values are in most cases between 1.d-4 and 1.d-5.\n"
"     RANGE: [1.e-20, 1.]\n"
"     default: 1.e-4\n"
"     fast: 1.e-3\n"
"     accurate: 1.e-5\n"
"   itermax:\n"
"     COMMENT: Max. iterations of wfn. opt. steps\n"
"     DESCRIPTION: >\n"
"       Maximum number of gradient evaluations for a single cycle in a wavefunction optimization and maximum number of\n"
"       cycles. At the end of each cycle a subspace diagonalization is done which helps in cases where one has near\n"
"       degeneracy between the HOMO and LUMO orbitals. 50 are usually sufficient.\n"
"     RANGE: [0, 10000]\n"
"     default: 50\n"
"   itermin:\n"
"     COMMENT: Minimal iterations of wfn. optimized steps\n"
"     RANGE: [0, 10000]\n"
"     default: 0\n"
"   nrepmax:\n"
"     COMMENT: Max. number of re-diag. runs\n"
"     DESCRIPTION: >\n"
"       Maximum number of gradient evaluations for a single cycle in a wavefunction optimization and maximum number of\n"
"       cycles. At the end of each cycle a subspace diagonalization is done which helps in cases where one has near\n"
"       degeneracy between the HOMO and LUMO orbitals. 50 are usually sufficient.\n"
"     RANGE: [0, 1000]\n"
"     default: 1\n"
"     accurate: 10\n"
"   ncong:\n"
"     COMMENT: No. of CG it. for preconditioning eq.\n"
"     DESCRIPTION: |\n"
"       ncong gives the number of iterations in the solution of the preconditioning equation. For free boundary\n"
"       conditions 5 is a good value whereas for other boundary condition a value from 0 to 2 is in general sufficient.\n"
"\n"
"       Large values of ncong lead to a smaller number of iterations in the wavefunction optimization and better\n"
"       forces, but each iteration is more costly.\n"
"       So an optimal compromise value has to be found.\n"
"     RANGE: [0, 20]\n"
"     default: 6\n"
"   idsx:\n"
"     COMMENT: Wfn. diis history\n"
"     DESCRIPTION: |\n"
"       idsx gives the history length of the DIIS convergence\n"
"       acceleration in the wavefunction optimization. 6 is usually a good value for fast convergence.\n"
"\n"
"       In case of convergence problems it can be advantageous to switch off DIIS by setting idsx = 0. The memory\n"
"       requirements grow considerably with large values of idsx.\n"
"\n"
"       If memory is the limiting factor one has to choose idsx smaller than the value which gives the fastest\n"
"       convergence.\n"
"       The bigdft-tool tool can be used to predict the memory requirements for different choices of idsx.\n"
"     RANGE: [0, 15]\n"
"     default: 6\n"
"     low_memory: 2\n"
"   dispersion:\n"
"     COMMENT: Dispersion correction potential (values 1,2,3,4,5), 0=none\n"
"     DESCRIPTION: |\n"
"       A non-zero values activates an empirical add-on treatment of dispersion effects.\n"
"\n"
"       The values 1, 2 and 3 specify different switching on functions using the convention.\n"
"     RANGE: [0, 5]\n"
"     default: 0\n"
"   inputpsiid:\n"
"     COMMENT: Input guess wavefunctions\n"
"     DESCRIPTION: |\n"
"       Specifies how the input guess wavefunction is generated\n"
"       - -2 Random numbers are used as input guess.\n"
"       his is of course a poor input guess which will need many iterations of the wavefunction optimization\n"
"       and might even lead to divergences.\n"
"\n"
"        - -1 The input wavefunction is imported from the CP2K code which uses Gaussian functions.\n"
"        The basis set should be contained in a file named gaubasis.dat whereas the coefficients should appear\n"
"        in the gaucoeff.dat file. Both files are the output files of CP2K code. See the H2O-CP2K test for an example.\n"
"\n"
"        - 0 A subspace diagonalization in a minimal atomic basis set is used. This input guess should be\n"
"        used in general if one starts a new calculation.\n"
"\n"
"        - 1 The previously calculated wavefunctions (for instance from the previous geometry optimization step)\n"
"        are used as input guess. Setting InputPsiId to this value does only make sense from within a main program\n"
"        where call_cluster() subroutine was called previously. The old wavefunction is passed to the new call via the\n"
"        data structure restart.\n"
"\n"
"        - 2 The input wavefunction is read from the wavefunction. files which contain all the scaling\n"
"        function and wavelet coefficients. In case some parameters such as hgrids or crmult have changed, compared to\n"
"        the previous run, the wavefunctions will be also transformed to the new parameter set. Use also this value to\n"
"        restart from ETSF file format wavefunction-etsf.nc.\n"
"\n"
"        - 10 The same as 0 but activates the ’Gaussian help’ after convergence the wavefunctions are projected\n"
"        onto the localised basis set used for the input guess, and Mulliken Charge Population Analysis (MCPA) is performed on\n"
"        this basis. The user has the possibility to perform MCPA with separate basis set (functionality to be added).\n"
"\n"
"        - 11 Restart with Gaussian approximation contained in the ’restart’ data structure.\n"
"\n"
"        - 12 The input wavefunction is read from the wavefunctions.gau file, which contains an approximation in\n"
"        a minimal Gaussian basis set of the previously calculated wavefunctions.\n"
"\n"
"        - 100 Use Linear Atomic Orbitals\n"
"\n"
"        - 101 Restart for Linear\n"
"\n"
"        - 102 Linear on disk\n"
"     default: 0\n"
"     EXCLUSIVE:\n"
"       -1000: Empty\n"
"       -2:    Random\n"
"       -1:    CP2K\n"
"       0:     LCAO\n"
"       1:     Wvl. in memory\n"
"       2:     Wvl. on disk\n"
"       3:     Plane-waves on disk\n"
"       10:    LCAO + gauss.\n"
"       11:    Gauss. in memory\n"
"       12:    Gauss. on disk\n"
"       100:   Linear AO\n"
"       101:   Linear restart\n"
"       102:   Linear on disk\n"
"     #Useful shortcuts\n"
"     LCAO: 0\n"
"     RANDOM: -2\n"
"     #Here follow two shortcuts for linear\n"
"     linear: 100\n"
"     linear_restart: 102\n"
"   projection:\n"
"     COMMENT: Projector construction method\n"
"     DESCRIPTION: |\n"
"       Specifies how the projectors are expressed in Daubechies from atomic description\n"
"     default: gaussian\n"
"     EXCLUSIVE:\n"
"       gaussian: tensor product of 1D separable gaussians\n"
"       radial: real space collocation of radial functions\n"
"       radial+mp: as radial, but using multipole preserving techniques\n"
"  #  output_wf:\n"
"  #    COMMENT: Output of the support functions\n"
"  #    DESCRIPTION: |\n"
"  #      controls the output of wavefunctions on disk at the end of the calculation\n"
"   #\n"
"  #         - output_wf = 0, Do not write the wavefunctions to disk.\n"
"   #\n"
"  #         - output_wf = 1, The output wavefunctions are written at the end of the wavefunction optimization into\n"
"  #         plain text files. If InputPsiId was greater than 10 the wavefunction will be written in the Gaussian\n"
"  #         approximation into a single wavefunctions.gau file otherwise into wavefunction. files. Writing a\n"
"  #         wavefunction. file for each orbital can take a considerable amount of time and disk space.\n"
"   #\n"
"  #         - output_wf = 2, The output wavefunctions are written at the end of the wavefunction optimization into\n"
"  #         Fortran binary files. This format is not portable between compilers and machines.\n"
"   #\n"
"  #         - output_wf = 3, The output wavefunctions are written at the end of the wavefunction optimization into\n"
"  #         ETSF binary files. This format is portable between compilers and machines since based on NetCDF. Parallel\n"
"  #         IO are taken into account.\n"
"  #    default: 0\n"
"  #    EXCLUSIVE:\n"
"  #      0: None\n"
"  #      1: Wavelet, Plain text\n"
"  #      2: Wavelet, Fortran binary\n"
"  #      3: Wavelet, ETSF file format\n"
"  #      #10: None\n"
"  #      #11: Wavelet and Interpolating Scaling Function, Plain text\n"
"  #      #12: Wavelet and Interpolating Scaling Function, Fortran binary\n"
"  #      #13: Wavelet and Interpolating Scaling Function, ETSF file format\n"
"  #      #20: None\n"
"  #      #21: Wavelet and Interpolating Scaling Function (wavefunction), Plain text\n"
"  #      #22: Wavelet and Interpolating Scaling Function (wavefunction), Fortran binary\n"
"  #      #23: Wavelet and Interpolating Scaling Function (wavefunction), ETSF file format\n"
"   output_denspot:\n"
"     COMMENT: Output of the density or the potential\n"
"     DESCRIPTION: |\n"
"       controls the output of density and pontentials on disk at the end of the calculation\n"
"\n"
"       - output_denspot = 0, No output density is written.\n"
"\n"
"       - output_denspot = 1, Output electronic density is written in the .pot format of V_Sim into the file\n"
"       electronic_density.pot (deprecated, use 11 instead).\n"
"\n"
"       - output_denspot = 2, In addition to the electronic density, the potential (local_potential.pot) and its\n"
"       components (external_potential.pot and Hartree_potential.pot) are also output in plain text .pot\n"
"       files (deprecated, use 12 instead).\n"
"\n"
"       - output_denspot = 11, Same as output_denspot = 1, but files are written in ETSF file format\n"
"       (portable binary format based on NetCDF).\n"
"\n"
"       - output_denspot = 12, Same as output_denspot = 2, but files are written in ETSF file format\n"
"       (portable binary format based on NetCDF).\n"
"\n"
"       - output_denspot = 21, Same as output_denspot = 1, but files are written in .cube file format\n"
"       (plain text).\n"
"\n"
"       - output_denspot = 22, Same as output_denspot = 2, but files are written in .cube file format\n"
"       (plain text).\n"
"     default: 0\n"
"     EXCLUSIVE:\n"
"       0:  None\n"
"       1:  Density in plain text\n"
"       2:  Density and potentials in plain text\n"
"       11: Density in ETSF file format\n"
"       12: Density and potentials in ETSF file format\n"
"       21: Density in cube file format\n"
"       22: Density and potentials in cube file format\n"
"   rbuf:\n"
"     COMMENT: Length of the tail (AU)\n"
"     description: |\n"
"       Far reaching tails of the wavefunctions decaying into the vacuum are added in a perturbative treatment if\n"
"       the variable rbuf is set to a strictly positive value.\n"
"       This allows to do a calculation with some moderate value of crmult and then to extrapolate to the limit of\n"
"       large crmult. This procedure is not variational and gives too low energies.\n"
"\n"
"       The true energy is in between the two energies and in general much closer to the extrapolated energy.\n"
"\n"
"       This procedure can also be used to judge whether the chosen value of crmult is large enough for a certain\n"
"       required precision. rbuf gives the amount by which the radii for the coarse resolution region are increased\n"
"       in atomic units. ncongt gives the number of iterations used in the perturbation calculation.\n"
"\n"
"       Reasonable values for ncongt are around 30.\n"
"     RANGE: [0., 10.]\n"
"     default: 0.\n"
"     #Here shortcut\n"
"     with tails: 5.\n"
"   ncongt:\n"
"     COMMENT: No. of tail CG iterations\n"
"     DESCRIPTION: |\n"
"       Far reaching tails of the wavefunctions decaying into the vacuum are added in a perturbative treatment if\n"
"       the variable rbuf is set to a strictly positive value.\n"
"       This allows to do a calculation with some moderate value of crmult and then to extrapolate to the limit of\n"
"       large crmult. This procedure is not variational and gives too low energies.\n"
"\n"
"       The true energy is in between the two energies and in general much closer to the extrapolated energy.\n"
"\n"
"       This procedure can also be used to judge whether the chosen value of crmult is large enough for a certain\n"
"       required precision. rbuf gives the amount by which the radii for the coarse resolution region are increased\n"
"       in atomic units. ncongt gives the number of iterations used in the perturbation calculation.\n"
"\n"
"       Reasonable values for ncongt are around 30.\n"
"     RANGE: [0, 50]\n"
"     default: 30\n"
"   norbv:\n"
"     COMMENT: Davidson subspace dimension (No. virtual orbitals)\n"
"     DESCRIPTION: |\n"
"        Determine the dimension of the subspace i.e. the number of virtual orbitals\n"
"        to calculate via a Davidson diagonalisation.\n"
"        Usually unoccupied orbitals are not calculated since they are not needed for the total energy and other\n"
"        physical properties of the electronic ground state.\n"
"\n"
"        Putting norbv to a non-zero value will result in the calculation of norbv virtual orbitals in a postprocessing\n"
"        routine after the occupied orbitals have been calculated, which uses Davidson iterative treatment.\n"
"        nvirt (< norbv) determines to the number of converged orbitals during the convergence process which is taking\n"
"        care of during the Davidson iterations.\n"
"        nplot of these orbitals will be written in the virtual.*.pot files and nplot (if that many exist) of the highest\n"
"        occupied orbitals will be written in the orbital.*.pot file. The Kohn Sham eigenvalues are written in the\n"
"        ordinary output file.\n"
"     RANGE: [-9999, 9999]\n"
"     default: 0\n"
"   nvirt:\n"
"     COMMENT: No. of converged virtual orbs (< norbv)\n"
"     DESCRIPTION: |\n"
"        Determine the dimension of the subspace i.e. the number of virtual orbitals\n"
"        to be converged via a Davidson diagonalisation. (should be < norbv)\n"
"        Usually unoccupied orbitals are not calculated since they are not needed for the total energy and other\n"
"        physical properties of the electronic ground state.\n"
"\n"
"        Putting norbv to a non-zero value will result in the calculation of norbv virtual orbitals in a postprocessing\n"
"        routine after the occupied orbitals have been calculated, which uses Davidson iterative treatment.\n"
"        nvirt (< norbv) determines to the number of converged orbitals during the convergence process which is taking\n"
"        care of during the Davidson iterations.\n"
"        nplot of these orbitals will be written in the virtual.*.pot files and nplot (if that many exist) of the highest\n"
"        occupied orbitals will be written in the orbital.*.pot file. The Kohn Sham eigenvalues are written in the\n"
"        ordinary output file.\n"
"     RANGE: [0, 9999]\n"
"     default: 0\n"
"   nplot:\n"
"     COMMENT: No. of plotted orbs\n"
"     DESCRIPTION: |\n"
"        Number of orbitals to be plotted\n"
"\n"
"        Determine the dimension of the subspace i.e. the number of virtual orbitals\n"
"        to calculate via a Davidson diagonalisation.\n"
"\n"
"        Usually unoccupied orbitals are not calculated since they are not needed for the total energy and other\n"
"        physical properties of the electronic ground state.\n"
"\n"
"        Putting norbv to a non-zero value will result in the calculation of norbv virtual orbitals in a postprocessing\n"
"        routine after the occupied orbitals have been calculated, which uses Davidson iterative treatment . nplot\n"
"        of these orbitals will be written in the virtual.*.pot files and nplot (if that many exist) of the highest\n"
"        occupied orbitals will be written in the orbital.*.pot file The Kohn Sham eigenvalues are written in the\n"
"        ordinary output file. nvirt corresponds to the actual number of orbitals the convergence process is taking\n"
"        care of during the Davidson iterations.\n"
"     RANGE: [0, 9999]\n"
"     default: 0\n"
"   gnrm_cv_virt:\n"
"     COMMENT: Convergence criterion gradient for virtual orbitals\n"
"     DESCRIPTION: |\n"
"       Norm of the gradient for virtual orbitals convergence.\n"
"       Calculates the results on the first nvirt orbitals\n"
"     RANGE: [1.e-20, 1.]\n"
"     default: 1.e-4\n"
"     fast: 1.e-3\n"
"     accurate: 1.e-5\n"
"   itermax_virt:\n"
"     COMMENT: Max. iterations of wfn. opt. steps for virtual orbitals\n"
"     DESCRIPTION: |\n"
"       Maximum number of gradient evaluations for a cycle of a wavefunction optimization\n"
"       for the virtual orbitals.\n"
"     RANGE: [0, 10000]\n"
"     default: 50\n"
"   disablesym:\n"
"     COMMENT: Disable the symmetry detection\n"
"     DESCRIPTION: |\n"
"       Disable the symmetry detection (by default No except for isolated systems).\n"
"\n"
"       By default symmetries are used to update the density in periodic boundary condition calculations, and surface\n"
"       boundary conditions also.\n"
"     default: No\n"
"   #   solvent:\n"
"   #     COMMENT: Electrostatic environment for Poisson Equation\n"
"   #     DESCRIPTION: Define the method to set the dielectric function as a function of the position\n"
"   #     EXCLUSIVE:\n"
"   #       vacuum: No Solvent method, ordinary Poisson Equation\n"
"   #       rigid:  Rigid (smoothed) cavity around the atoms, given as superposition of spheres\n"
"   #       sccs: Self-Consistent Continuum Solvation (accordig to the model of Andreussi et. al JCP 136, 064102 (2012)\n"
"   #     default: vacuum\n"
"   external_potential:\n"
"     COMMENT: Multipole moments of an external potential\n"
"     DESCRIPTION: Give the multipole moments up to quadrupoles\n"
"     default: 0.0\n"
"   calculate_strten:\n"
"     COMMENT: Boolean to activate the calculation of the stress tensor. Might be set to No for performance reasons\n"
"     DESCRIPTION: |\n"
"      In some cases, for example when doing MD for fixed volume systems, it might be useful to disable the\n"
"      calculation of the stress tensor, to save computational time.\n"
"     default: Yes\n"
"   magnetic_torque:\n"
"     COMMENT: Boolean to activate the calculation of the direction of the atomic magnetization which lowers the total energy\n"
"     default: No\n"
"     CONDITION:\n"
"       MASTER_KEY: nspin\n"
"       WHEN:\n"
"       - 4\n"
"   spin_dynamics:\n"
"     COMMENT: Boolean to activate the calculation of the direction of the atomic magnetization which lowers the total energy\n"
"     default: No\n"
"     CONDITION:\n"
"       MASTER_KEY: magnetic_torque\n"
"       WHEN:\n"
"       - Yes\n"
"   plot_mppot_axes:\n"
"     COMMENT: Plot the potential generated by the multipoles along axes through this point. Negative values mean no plot.\n"
"     DESCRIPTION: |\n"
"      If present, the potential generated by the multipoles will be written in the .cube format and also be plotted\n"
"      along axes through the specified point. If no plot is desired, the values should be negative.\n"
"     default: [-1,-1,-1]\n"
"   plot_pot_axes:\n"
"     COMMENT: Plot the potential along axes through this point. Negative values mean no plot.\n"
"     DESCRIPTION: |\n"
"      If present and the chosen plotting format is .cube, the potential will be written in the .cube format and also be plotted\n"
"      along axes through the specified point. If no plot is desired, the values should be negative.\n"
"     default: [-1,-1,-1]\n"
"     #CONDITION:\n"
"     #  MASTER_KEY: output_denspot\n"
"     #  WHEN:\n"
"     #  - 22\n"
"   occupancy_control:\n"
"    COMMENT: Dictionary of the atomic matrices to be applied for a given iteration number\n"
"    DESCRIPTION: |\n"
"     The atomic species or the atoms written as Atom No are specified, followed by a dictionary\n"
"     of the shells for which the density matrix can be controlled. The Value of the matrix can be specified either as\n"
"     a trace or as explicit matrix elements.\n"
"    default: None\n"
"   itermax_occ_ctrl:\n"
"     COMMENT: Number of iterations of occupancy control scheme. Should be between itermin and itermax\n"
"     default: 0\n"
"   nrepmax_occ_ctrl:\n"
"     COMMENT: Number of re-diagonalizations of occupancy control scheme.\n"
"     default: 1\n"
"   alpha_hf:\n"
"     COMMENT: Part of the exact exchange contribution for hybrid functionals\n"
"     DESCRIPTION: |\n"
"       By default .25 for PBE0, and .2 for B3LYP\n"
"     RANGE: [0.0,1.0]\n"
"     default: -1.0\n"
"#output parameters, decide what to dump of write to file\n"
" output:\n"
"  DESCRIPTION: Output parameters, control the information that will be printed out\n"
"  verbosity:\n"
"    COMMENT: Verbosity of the output\n"
"    EXCLUSIVE:\n"
"      0: Very low\n"
"      1: Low\n"
"      2: Medium\n"
"      3: High\n"
"    default: 2\n"
"  atomic_density_matrix:\n"
"    COMMENT: Dictionary of the atoms for which the atomic density matrix has to be plotted\n"
"    DESCRIPTION: |\n"
"     The atomic species or the atoms written as Atom No are specified, followed by a list\n"
"     of the shells for which the density matrix can be plotted\n"
"    default: None\n"
"  sdos:\n"
"    COMMENT: Spatially-resolved density of states\n"
"    DESCRIPTION: |\n"
"     Plots the density of states by calculating the value of psi*Hpsi per each KS orbital,\n"
"     resolved among x,y or z.\n"
"    default: No\n"
"  orbitals:\n"
"    COMMENT: Write KS orbitals in the full simulation domain (might take lot of disk space!)\n"
"    default: None\n"
"    EXCLUSIVE:\n"
"       None: No output\n"
"       text: Output in wavelet basis\n"
"       binary: Binary file output\n"
"       text_with_densities: In addition output wavefunction densities in interpolating scaling functions (cube file)\n"
"       text_with_cube: In addition output wavefunctions in interpolating scaling functions (cube file)\n"
"       etsf: ETSF file format\n"
"  outputpsiid:\n"
"    COMMENT: basename of the output wavefunctions files written in the data directory\n"
"    default: wavefunction\n"
"    DESCRIPTION: |\n"
"      Define the basename of all the files associated to the output of the wavefunctions.\n"
"      Can be reused in the inputpsiid input parameter.\n"
"  coupling_matrix:\n"
"    COMMENT: kind of the coupling matrix which have to be plotted\n"
"    DESCRIPTION: |\n"
"       Useful to separate the various contributions of the couling matrix. May be plotted exclusively, separately or in complete form.\n"
"    default: complete\n"
"    EXCLUSIVE:\n"
"       complete: dump the total coupling matrix\n"
"       rpa: only dump the RPA term\n"
"       fxc: only dump the fxc term\n"
"  #potentials:\n"
"  #  format: CUBE\n"
"  #  frequence: iteration, SCF\n"
"\n"
"#K points parameters\n"
" kpt:\n"
"   DESCRIPTION: K points defined in the Brillouin zone for periodic system calculations\n"
"   method:\n"
"     COMMENT: K-point sampling method\n"
"     EXCLUSIVE:\n"
"       auto:   Based on kptrlen\n"
"       mpgrid: Monkhorst-Pack\n"
"       manual: Based on raw coordinates\n"
"     default:  manual\n"
"   kptrlen:\n"
"     COMMENT: Equivalent length of K-space resolution (Bohr)\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - auto\n"
"     RANGE: [0, 1e4]\n"
"     default: 0.\n"
"   ngkpt:\n"
"     COMMENT: No. of Monkhorst-Pack grid points\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - mpgrid\n"
"     RANGE: [1, 10000]\n"
"     default: [1, 1, 1]\n"
"   shiftk:\n"
"     COMMENT: Grid shifts\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - mpgrid\n"
"     default: [[0., 0., 0.]]\n"
"   kpt:\n"
"     COMMENT: Kpt coordinates\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - manual\n"
"     default:\n"
"     -  [0., 0., 0.]\n"
"   wkpt:\n"
"     COMMENT: Kpt weights\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - manual\n"
"     RANGE: [0., 1e4]\n"
"     default:\n"
"     - 1.\n"
"   bands:\n"
"     COMMENT: For doing band structure calculation\n"
"     default: No\n"
"   iseg:\n"
"     COMMENT: Points for each segment of the Brillouin zone path\n"
"     CONDITION:\n"
"       MASTER_KEY: bands\n"
"       WHEN:\n"
"       - Yes\n"
"     default:\n"
"     - 1\n"
"   kptv:\n"
"     COMMENT: K points defining each segment of the Brillouin zone path\n"
"     CONDITION:\n"
"       MASTER_KEY: bands\n"
"       WHEN:\n"
"       - Yes\n"
"     default:\n"
"     -  [0., 0., 0.]\n"
"     -  [0.5, 0.5, 0.5]\n"
"   ngranularity:\n"
"     COMMENT: No. of points done for each group\n"
"     CONDITION:\n"
"       MASTER_KEY: bands\n"
"       WHEN:\n"
"       - Yes\n"
"     RANGE: [1, 1000]\n"
"     default: 1\n"
"   band_structure_filename:\n"
"     COMMENT: Filename given the local potential for band structure calculations\n"
"     CONDITION:\n"
"       MASTER_KEY: bands\n"
"       WHEN:\n"
"       - Yes\n"
"     default: ''\n"
"\n"
"#Geometry Optimisation  and ABINIT Molecular Dynamics parameters\n"
" geopt:\n"
"   DESCRIPTION: |\n"
"          This character string specifies the method used for the geometry optimization.\n"
"          - VSSD Variable Stepsize Steepest Descent method.\n"
"          - SDCG A combination of Steepest Descent and Conjugate Gradient.\n"
"          - LBFGS Limited Memory BFGS.\n"
"\n"
"     #    - BFGS Preconditioned steepest descent with energy feedback. A preconditioning matrix is build\n"
"     #    up according to the BFGS algorithm. The initial Hessian is a diagonal matrix where the\n"
"     #    diagonal elements are the inverse of the step size. This method is usually the most efficient\n"
"     #    one.\n"
"     #\n"
"     #     - PBFGS Same as BFGS except that an initial Hessian is obtained from a force field. Force field\n"
"     #     parameters are only available for systems consisting of H,C,N,O. For such systems the method\n"
"     #     is the most efficient one in general\n"
"     #\n"
"     #    - FIRE The Fast Inertial Relaxation Engine as described by Bitzek et al.[1] with optimized\n"
"     #    parameters. An additional line is required in the input.geopt file with two parameters\n"
"     #    initial time step dtinit and the maximum time step dtmax.\n"
"     ###    A good starting point to estimate dtmax is by using dtmax=0.25 pi sqrt(betax_(VSSD)) and\n"
"     #    dtinit=0.5dtmax, and gradually increasing dtmax. betax is not used.\n"
"\n"
"     #    - AB6MD The molecular dynamic routines from ABINIT 6.\n"
"     #\n"
"     #    - DIIS a Direct Inversion of Iterative Subspace. In that case, an additional parameter is required\n"
"     #    qqwith the history length (default is 4) on the line of betax.\n"
"   method:\n"
"     COMMENT: Geometry optimisation method\n"
"     EXCLUSIVE:\n"
"       none:   No geometry optimization\n"
"       SDCG:   A combination of Steepest Descent and Conjugate Gradient\n"
"       VSSD:   Variable Stepsize Steepest Descent method\n"
"       LBFGS:  Limited-memory BFGS\n"
"       BFGS:   Broyden-Fletcher-Goldfarb-Shanno\n"
"       PBFGS:  Same as BFGS with an initial Hessian obtained from a force field\n"
"       AB6MD:  Molecular dynamics from ABINIT\n"
"       DIIS:   Direct inversion of iterative subspace\n"
"       FIRE:   Fast Inertial Relaxation Engine as described by Bitzek et al.\n"
"       SOCK:   Socket communication\n"
"       NEB:    Nudged Elastic Band\n"
"       SBFGS:  SQNM minimizer, keyword deprecated, will be replaced by SQNM in future release\n"
"       SQNM:   Stabilized quasi-Newton minimzer\n"
"       MD: Molecular dynamics, NVT (Nose-Hoover Chains) or NVE ensemble\n"
"       LOOP:   Do nothing except looping, forces should be changed externally\n"
"     default: none\n"
"   ncount_cluster_x:\n"
"     COMMENT: Maximum number of force evaluations\n"
"     DESCRIPTION: |\n"
"       Maximum number of force evaluations to be used for the geometry optimization.\n"
"     RANGE: [0, 2000]\n"
"     PROFILE_FROM: method\n"
"     default: 50\n"
"     none: 1\n"
"   sockinet:\n"
"     COMMENT: Socket either unix (0) or tcp (1)\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SOCK\n"
"     RANGE: [0, 1]\n"
"     default: 1\n"
"   sockport:\n"
"     COMMENT: Socket port number\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SOCK\n"
"     RANGE: [0, 100000]\n"
"     default: 2121\n"
"   sockhost:\n"
"     COMMENT: Socket host name\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SOCK\n"
"     default: 127.0.0.1\n"
"   frac_fluct:\n"
"     COMMENT: Fraction of force fluctuations. Stop if fmax < forces_fluct*frac_fluct\n"
"     DESCRIPTION: |\n"
"       Convergence criteria for the geometry optimization.\n"
"\n"
"       The geometry optimization stops either if the norm of the individual forces acting on any atom in the\n"
"       system is smaller than forcemax or if the forces get noisy.\n"
"       Noise is present because of the underlying integration grid. The parameter frac_fluct specifies how\n"
"       small the forces should become compared to the noise level to stop the geometry optimization.\n"
"\n"
"       Values in between 1 and 10 is are reasonable for this parameter. A value of 2 means that the geometry\n"
"       optimization will stop when the largest atomic force is comparable to the 2 times the average noise\n"
"       in the forces themselves.\n"
"       For values of frac_fluct smaller than 1 one can under certain circumstances obtain better relaxed\n"
"       geometries but one risks that the geometry optimization will not converge since the forces are too\n"
"       noisy.\n"
"\n"
"       In such a case one should closely monitor the progress of the geometry optimization by looking at the\n"
"       posout. files which are written at each step of the geometry optimization and at the data/geopt.mon\n"
"       file.\n"
"     RANGE: [0., 10.]\n"
"     default: 1.\n"
"   forcemax:\n"
"     COMMENT: Max forces criterion when stop\n"
"     DESCRIPTION: |\n"
"       Convergence criteria for the geometry optimization.\n"
"\n"
"       The geometry optimization stops either if the norm of the individual forces acting on any atom in the\n"
"       system is smaller than forcemax or if the forces get noisy.\n"
"       Noise is present because of the underlying integration grid. The parameter frac_fluct specifies how\n"
"       small the forces should become compared to the noise level to stop the geometry optimization.\n"
"\n"
"       Values in between 1 and 10 is are reasonable for this parameter. A value of 2 means that the geometry\n"
"       optimization will stop when the largest atomic force is comparable to the 2 times the average noise\n"
"       in the forces themselves.\n"
"       For values of frac_fluct smaller than 1 one can under certain circumstances obtain better relaxed\n"
"       geometries but one risks that the geometry optimization will not converge since the forces are too\n"
"       noisy.\n"
"\n"
"       In such a case one should closely monitor the progress of the geometry optimization by looking at the\n"
"       posout. files which are written at each step of the geometry optimization and at the data/geopt.mon\n"
"       file.\n"
"     RANGE: [0., 10.]\n"
"     PROFILE_FROM: method\n"
"     default: 0.\n"
"     NEB: 5e-3\n"
"   randdis:\n"
"     COMMENT: Random displacement amplitude\n"
"     DESCRIPTION: |\n"
"       This parameter allows to add random displacements with amplitude randdis to the atomic positions in\n"
"       the input file posinp.\n"
"\n"
"       This can for instance be useful to break degeneracies (which would lead to convergence problems in\n"
"       the wavefunction optimization) in highly symmetric structures.\n"
"     RANGE: [0., 10.]\n"
"     default: 0.\n"
"   betax:\n"
"     COMMENT: Stepsize for the geometry optimization\n"
"     DESCRIPTION: |\n"
"       This is the stepsize for the geometry optimization.\n"
"\n"
"       This stepsize is system dependent and it has therefore to be determined for each system.\n"
"       If the VSSD method is used one can start with a small stepsize of around 1 and VSSD will suggest then\n"
"       a better value for {betax} in the last line of the data/geopt.mon file. Whether the stepsize is correct\n"
"       can also be seen from the data/geopt.mon output of the SDCG method.\n"
"\n"
"       In this case the average stepsize in terms of betax should be around 4 (after a brief initial period\n"
"       where it is around 8).\n"
"       In contrast to the SDCG method, the BFGS method is not very sensitive to the correct stepsize but\n"
"       nevertheless one should try to find reasonable values also in this case.\n"
"\n"
"       SQNM: For many systems 1 is a good stepsize. Some systems can be minimized more efficiently with\n"
"       a larger stepsize.\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SDCG\n"
"       - VSSD\n"
"       - LBFGS\n"
"       - BFGS\n"
"       - PBFGS\n"
"       - DIIS\n"
"       - FIRE\n"
"       - NEB\n"
"       - SBFGS\n"
"       - SQNM\n"
"       - none\n"
"     PROFILE_FROM: method\n"
"     RANGE: [0., 100.]\n"
"     default: 4.\n"
"     DIIS: 2.\n"
"     NEB: 0.5\n"
"   history:\n"
"     COMMENT: History for DIIS method\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - DIIS\n"
"     RANGE: [0, 100]\n"
"     default: 4\n"
"   dtinit:\n"
"     COMMENT: Initial time step for the FIRE method\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - FIRE\n"
"     RANGE: [0, 1e4]\n"
"     default: 0.75\n"
"   dtmax:\n"
"     COMMENT: Maximal time step for the FIRE method\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - FIRE\n"
"     RANGE: [0, 1e4]\n"
"     default: 1.5\n"
"   nhistx:\n"
"     COMMENT: Maximum history length\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SBFGS\n"
"       - SQNM\n"
"     RANGE: [0, 1000]\n"
"     default: 10\n"
"   biomode:\n"
"     COMMENT: Enable mode for biomolecules.\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SBFGS\n"
"       - SQNM\n"
"     default: No\n"
"   beta_stretchx:\n"
"     COMMENT: Stepsize for steepest descent in stretching mode direction (only if in biomode)\n"
"     RANGE: [0., 100.]\n"
"     default: 5e-1\n"
"   maxrise:\n"
"     COMMENT: Maximum allowed rise in energy in one geopt step\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SBFGS\n"
"       - SQNM\n"
"     RANGE: [0., 1e10]\n"
"     default: 1e-6\n"
"   cutoffratio:\n"
"     COMMENT: If ratio of eigenvalues of displacement overlapmatrix is smaller than cutoffratio, the corresponding directions are not used\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SBFGS\n"
"       - SQNM\n"
"     RANGE: [0., 1.]\n"
"     default: 1e-4\n"
"   steepthresh:\n"
"     COMMENT: If fmax < steepthresh, geopt method uses steepest descent\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SBFGS\n"
"       - SQNM\n"
"     RANGE: [0., 1e10]\n"
"     default: 0.1\n"
"   trustr:\n"
"     COMMENT: Maximum displacement of single atom\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - SBFGS\n"
"       - SQNM\n"
"     RANGE: [0., 1e10]\n"
"     default: 0.5\n"
"   ionmov:\n"
"     COMMENT: Movement ion method\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - AB6MD\n"
"     EXCLUSIVE:\n"
"       6:  Simple velocity-Verlet molecular dynamic\n"
"       7:  Quenched molecular dynamic\n"
"       8:  Nose-Hoover thermostat\n"
"       9:  Langevin dynamic\n"
"       12: Isokinetic ensemble molecular dynamics\n"
"       13: Iosthermal/isenthalpic ensemble\n"
"     default: 6\n"
"   dtion:\n"
"     COMMENT: Time step - Atomic Units (20.670689 AU=0.5 fs)\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - AB6MD\n"
"     RANGE: [0, 1e3]\n"
"     default: 20.670689\n"
"   mditemp:\n"
"     COMMENT: Temperature of molecular dynamics\n"
"     CONDITION:\n"
"       MASTER_KEY: ionmov\n"
"       WHEN: [6, 8, 9, 12, 13]\n"
"#       - 6\n"
"     RANGE: [0, 1e9]\n"
"     default: 300.\n"
"   mdftemp:\n"
"     COMMENT: Final temperature of molecular dynamics\n"
"     CONDITION:\n"
"       MASTER_KEY: ionmov\n"
"       WHEN: [8, 9, 12, 13]\n"
"     RANGE: [0, 1e9]\n"
"     default: 300.\n"
"   noseinert:\n"
"     COMMENT: Thermostat inertia coefficient for Nose_Hoover dynamics\n"
"     CONDITION:\n"
"       MASTER_KEY: ionmov\n"
"       WHEN:\n"
"       - 8\n"
"     RANGE: [0, 1e9]\n"
"     default: 1e5\n"
"   friction:\n"
"     COMMENT: Friction coefficient for Langevin dynamics\n"
"     CONDITION:\n"
"       MASTER_KEY: ionmov\n"
"       WHEN:\n"
"       - 9\n"
"     RANGE: [0, 1e5]\n"
"     default: 1e-3\n"
"   mdwall:\n"
"     COMMENT: Distance in bohr where atoms can bounce for Langevin dynamics\n"
"     CONDITION:\n"
"       MASTER_KEY: ionmov\n"
"       WHEN:\n"
"       - 9\n"
"     RANGE: [0, 1e5]\n"
"     default: 1e4\n"
"   qmass:\n"
"     COMMENT: Mass of each thermostat (isothermal/isenthalpic ensemble)\n"
"     CONDITION:\n"
"       MASTER_KEY: ionmov\n"
"       WHEN:\n"
"       - 13\n"
"     RANGE: [0, 1e9]\n"
"     default:\n"
"     - 0.\n"
"   bmass:\n"
"     COMMENT: Barostat masses (isothermal/isenthalpic ensemble)\n"
"     CONDITION:\n"
"       MASTER_KEY: ionmov\n"
"       WHEN:\n"
"       - 13\n"
"     RANGE: [0, 1e9]\n"
"     default: 10.\n"
"   vmass:\n"
"     COMMENT: Barostat masses (isothermal/isenthalpic ensemble)\n"
"     CONDITION:\n"
"       MASTER_KEY: ionmov\n"
"       WHEN:\n"
"       - 13\n"
"     RANGE: [0, 1e9]\n"
"     default: 1.\n"
"   climbing:\n"
"     COMMENT: Make the highest energy image to climb along the path\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - NEB\n"
"     default: No\n"
"   extrema_opt:\n"
"     COMMENT: Minimize the energy of extremum images during NEB optimization\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - NEB\n"
"     default: No\n"
"   neb_method:\n"
"     COMMENT: Image minimization method\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - NEB\n"
"     EXCLUSIVE:\n"
"       steepest_descent: steepest descent\n"
"       fletcher-reeves: Fletcher Reeves\n"
"       polak-ribiere: Polak Ribiere\n"
"       quick-min: Conjugate gradient\n"
"       damped-verlet: Verlet with damp\n"
"       sim-annealing: Simulation of annealing\n"
"     default: quick-min\n"
"   temp:\n"
"     COMMENT: Temperature for simulated annealing\n"
"     RANGE: [0.,1000.]\n"
"     default: 0.\n"
"     CONDITION:\n"
"       MASTER_KEY: neb_method\n"
"       WHEN:\n"
"       - sim-annealing\n"
"   damp:\n"
"     COMMENT: Damping factor\n"
"     RANGE: [0.,1000.]\n"
"     default: 1.\n"
"     CONDITION:\n"
"       MASTER_KEY: neb_method\n"
"       WHEN:\n"
"       - damped-verlet\n"
"   springs_k:\n"
"     COMMENT: Maximum and minimum value when changing spring constant during optimization\n"
"     RANGE: [0.,1000.]\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - NEB\n"
"     default: [0.05,0.1]\n"
"   fix_tol:\n"
"     COMMENT: Tolerance to apply on atom moves between extrema\n"
"     RANGE: [0,100]\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - NEB\n"
"     default: 1e-4\n"
"   nimg:\n"
"     COMMENT: Number of images\n"
"     RANGE: [3,100]\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN:\n"
"       - NEB\n"
"     default: 15\n"
"#Molecular Dynamics (BigDFT)\n"
" md:\n"
"   DESCRIPTION: Molecular dynamics parameters\n"
"   mdsteps:\n"
"     COMMENT: Number of MD steps\n"
"     DESCRIPTION: A value greater than 0 will activate molecular dynamics\n"
"     RANGE: [0, .inf ]\n"
"     default: 0\n"
"   print_frequency:\n"
"     COMMENT: Printing frequency for energy.dat and Trajectory.xyz files\n"
"     RANGE: [1, 999999]\n"
"     default: 1\n"
"   temperature:\n"
"     COMMENT: Initial temperature in Kelvin\n"
"     RANGE: [0, 9999999.0]\n"
"     default: 300.d0\n"
"   timestep:\n"
"     COMMENT: Time step for integration (in a.u.)\n"
"     RANGE: [0, 99999.0]\n"
"     default: 20.d0\n"
"   no_translation:\n"
"     COMMENT: Logical input to set translational correction\n"
"     default: No\n"
"   thermostat:\n"
"     COMMENT: Activates a thermostat for MD\n"
"     EXCLUSIVE:\n"
"       none: no thermostat\n"
"       nose_hoover_chain: Nose-Hoover Chain thermostat\n"
"     default: none\n"
"   nose_chain_length:\n"
"     COMMENT: Length of the Nose-Hoover Chains (integer input)\n"
"     RANGE: [2, 10]\n"
"     CONDITION:\n"
"       MASTER_KEY: thermostat\n"
"       WHEN:\n"
"       - nose_hoover_chain\n"
"     default: 3\n"
"   nose_mts_size:\n"
"     COMMENT: Multiple time steps used to integrate Nose-Hoover Chains equations of motion (integer input)\n"
"     RANGE: [1, 10]\n"
"     CONDITION:\n"
"       MASTER_KEY: thermostat\n"
"       WHEN:\n"
"       - nose_hoover_chain\n"
"     default: 1\n"
"   nose_yoshida_factor:\n"
"     COMMENT: Yoshida-Suzuki order to integrate the Nose-Hoover Chains equations of motion (integer input)\n"
"     EXCLUSIVE: [3, 5, 7]\n"
"     CONDITION:\n"
"       MASTER_KEY: thermostat\n"
"       WHEN:\n"
"       - nose_hoover_chain\n"
"     default: 7\n"
"   nose_frequency:\n"
"     COMMENT: Frequency in (cm^-1) corresponding to the inertia of the thermostat degrees of freedom\n"
"     RANGE: [1., .inf ]\n"
"     CONDITION:\n"
"       MASTER_KEY: thermostat\n"
"       WHEN:\n"
"       - nose_hoover_chain\n"
"     default: 3000.0\n"
"   wavefunction_extrapolation:\n"
"     COMMENT: Activates the wavefunction extrapolation for MD\n"
"     DESCRIPTION: A value greater than 0 activates wfn extrapolation\n"
"     EXCLUSIVE: [ 0, 1, 2, 4 ]\n"
"     default: 0\n"
"   always_from_scratch:\n"
"     COMMENT: When true, always restart wavefunctions from scratch to eliminate dissipation effects\n"
"     default: No\n"
"   restart_nose:\n"
"     COMMENT: Restart Nose Hoover Chain information from md.restart\n"
"     default: No\n"
"   restart_pos:\n"
"     COMMENT: Restart nuclear position information from md.restart\n"
"     default: No\n"
"   restart_vel:\n"
"     COMMENT: Restart nuclear velocities information from md.restart\n"
"     default: No\n"
"#Mixing parameters (do diagonalisation scheme instead of direct minimisation one)\n"
" mix:\n"
"   DESCRIPTION: |\n"
"     This file contains all the parameters required to use a density (or potential)\n"
"     mixing scheme for the self consistent field loop, which may imply a smearing\n"
"     of the occupancies for finite temperature DFT calculations.\n"
"     It was the former input.mix.\n"
"   iscf:\n"
"     COMMENT: Mixing scheme (default=0 i.e. direct minimization)\n"
"     DESCRIPTION: |\n"
"       An integer giving the mixing scheme and the mixing target.\n"
"       It follows the ABINIT convention. For values lower than 10, the potential is mixed, while\n"
"       for values greater than 10, the density is mixed. A recommended option is Pulay mixing\n"
"       (7 or 17 for potential or charge mixing, respectively)\n"
"     EXCLUSIVE:\n"
"       -1: Reserved, do not use it.\n"
"       0:  Direct minimization\n"
"       1:  Get the largest eigenvalue of the SCF cycle\n"
"       2:  Simple mixing of the potential\n"
"       3:  Anderson mixing of the potential\n"
"       4:  Anderson mixing of the potential based on the two previous iterations\n"
"       5:  CG based on the minim. of the energy with respect to the potential\n"
"       7:  Pulay mixing of the potential based on npulayit previous iterations\n"
"       12: Simple mixing of the density\n"
"       13: Anderson mixing of the density\n"
"       14: Anderson mixing of the density based on the two previous iterations\n"
"       15: CG based on the minim. of the energy with respect to the density\n"
"       17: Pulay mixing of the density\n"
"     default: 0\n"
"   itrpmax:\n"
"     COMMENT: Maximum number of diagonalisation iterations\n"
"     DESCRIPTION: >\n"
"       Maximum number of repeats for the mixing scheme. Each iteration implies an optimization\n"
"       of the wavefunctions at constant potential according to the settings in input.dft.\n"
"     RANGE: [0, 10000]\n"
"     default: 1\n"
"   rpnrm_cv:\n"
"     COMMENT: Stop criterion on the residue of potential or density\n"
"     DESCRIPTION: >\n"
"       The stop criterion on the residue of the potential or the density. Typical values are in\n"
"       the order of 1e-10 to 1e-12.\n"
"     RANGE: [0., 10.]\n"
"     default: 1.e-4\n"
"   norbsempty:\n"
"     COMMENT: No. of additional bands\n"
"     DESCRIPTION: |\n"
"       The maximum number of additional (initially unoccupied) bands considered.\n"
"       If a larger value is given, The maximum of orbitals available via the input guess\n"
"       procedure will be used automatically\n"
"     RANGE: [0, 10000]\n"
"     default: 0\n"
"   tel:\n"
"     COMMENT: Electronic temperature\n"
"     DESCRIPTION: |\n"
"       The electronic temperature in energy units (Ha).\n"
"     RANGE: [0., 1e6]\n"
"     default: 0.\n"
"   occopt:\n"
"     COMMENT: Smearing method\n"
"     DESCRIPTION: |\n"
"       The smearing function to determine the occupation. The conventions differ from those\n"
"       used by ABINIT. 1 = Gaussian, 2 = Fermi, 3 = Marzari (bumb minimization), 4 = Marzari\n"
"       (monotonic tail), 5 = Methfessel and Paxton.\n"
"     EXCLUSIVE:\n"
"       1: Error function smearing\n"
"       2: Normal Fermi distribution\n"
"       3: Marzari cold smearing a=-.5634 (bumb minimization)\n"
"       4: Marzari cold smearing with a=-.8165 (monotonic tail)\n"
"       5: Methfessel and Paxton (same as cold with a=0)\n"
"     default: 1\n"
"   alphamix:\n"
"     COMMENT: Multiplying factors for the mixing\n"
"     DESCRIPTION: |\n"
"       The closer this value gets to one, the more cautious will be the mixing of the charge or potential.\n"
"       In the case of straight mixing, it corresponds to alpha according to\n"
"       rho_{new} <-- alpha rho_{old} + (1-alpha) - rho_{new}\n"
"       alphamix must be larger than one half.\n"
"       A typical choice for metals is 0.95.\n"
"     RANGE: [0, 1.]\n"
"     default: 0.\n"
"   alphadiis:\n"
"     COMMENT: Multiplying factors for the electronic DIIS\n"
"     DESCRIPTION: |\n"
"       The DIIS multiplier used for preconditioning the wavefunctions at constant potential.\n"
"     RANGE: [0., 10.]\n"
"     default: 2.\n"
"\n"
"#Self-Interaction Correction\n"
" sic:\n"
"   DESCRIPTION: Self-Interaction Correction parameters\n"
"   sic_approach:\n"
"     COMMENT: SIC (self-interaction correction) method\n"
"     EXCLUSIVE:\n"
"       none: No self-interaction correction\n"
"       PZ:   Perdew-Zunger SIC scheme\n"
"       NK:   Non-Koopmans correction (Experimental)\n"
"     default: none\n"
"   sic_alpha:\n"
"     COMMENT: SIC downscaling parameter\n"
"     RANGE: [0, 1.]\n"
"     default: 0.\n"
"   sic_fref:\n"
"     COMMENT: Reference occupation fref (for Non-Koopman case only)\n"
"     CONDITION:\n"
"       MASTER_KEY: sic_approach\n"
"       WHEN:\n"
"       - NK\n"
"     RANGE: [0., 1.]\n"
"     default: 0.\n"
"#Time-Dependent DFT parameters\n"
" tddft:\n"
"   tddft_approach:\n"
"     COMMENT: Time-Dependent DFT method\n"
"     EXCLUSIVE:\n"
"       none: no tddft post-processing\n"
"       TDA: Casida's LR-TDDFT using Tamm-Dancoff Approximation\n"
"       full: Full Casida's LR-TDDFT\n"
"     default: none\n"
"   decompose_perturbation:\n"
"     COMMENT: Indicate the directory of the perturbation to be decomposed in the basis of empty states\n"
"     default: none\n"
"#overall run setup\n"
" mode:\n"
"   method:\n"
"     COMMENT: Run method of BigDFT call\n"
"     EXCLUSIVE:\n"
"       dft: Density Functional Theory (BigDFT)\n"
"       lj: Lennard-Jones force field\n"
"       tdpot: two-dimension test potential\n"
"       lensic: Lenosky Force Field for Si clusters\n"
"       lensib: Lenosky Force Field for Si bulk\n"
"       amber: AMBER Force Fields Package (requires AMBERtools)\n"
"       morse_bulk: Morse bulk\n"
"       morse_slab: Morse slab\n"
"       tersoff: Tersoff potential for C and Si\n"
"       alborz: Alireza Ghasemis ALBORZ package\n"
"       bmhtf: Born Mayer Huggins Tosi Fumi Potential\n"
"       cp2k: CP2K\n"
"       dftbp: DFTP+\n"
"       sw: Stillinger-Weber for Si\n"
"       bazant:  Bazant force for Si\n"
"       multi: Mixed approach where parts of the system are treated with different methods\n"
"       plugin: Use the signaling procedure profided by FUTILE\n"
"     default: dft\n"
"   plugin_id:\n"
"     COMMENT: Integer handle identifying the calculator associated to the mode plugin\n"
"     default: 1\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN: plugin\n"
"   add_coulomb_force:\n"
"     COMMENT: Boolean to add coulomb force on top of any of above selected force\n"
"     MASTER_KEY: method\n"
"     default: No\n"
"   nab_options:\n"
"     COMMENT: String to be passed to nab_init compiler in the case of ambertools compilation\n"
"     default: ntpr=1\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN: amber\n"
"   mm_paramset:\n"
"     COMMENT: Name of the hard-coded parameter set. Must be compatible to the chosen force field.\n"
"     default: default\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN: [lj, tdpot, lensic, lensib, amber, morse_bulk, morse_slab, tersoff, bmhtf, cp2k, dftbp]\n"
"   mm_paramfile:\n"
"     COMMENT: Name of parameter or input file for the selected method or code\n"
"     default: none\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN: [lj, tdpot, lensic, lensib, amber, morse_bulk, morse_slab, tersoff, bmhtf, cp2k, dftbp]\n"
"   sections:\n"
"     COMMENT: List of labelling subparts to divide the system into\n"
"     default: []\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN: [multi]\n"
"   section_buffer:\n"
"     COMMENT: Level of neighbours added to posinp sections\n"
"     default: 1\n"
"     RANGE: [0, 10]\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN: [multi]\n"
"   section_passivation:\n"
"     COMMENT: Level of neighbours added to posinp sections\n"
"     default: Yes\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN: [multi]\n"
"   sw_eqfactor:\n"
"     COMMENT: Factor on sigma value, rule the equilibrium bond length for the Stillinger-Weber potential\n"
"     default: 1.\n"
"     PBE_Si: 1.006445672 # 5.465 / 5.430\n"
"     LDA_Si: 0.994475138 # 5.400 / 5.430\n"
"     CONDITION:\n"
"       MASTER_KEY: method\n"
"       WHEN: [sw]\n"
"\n"
"#Performance parameters\n"
" perf:\n"
"   debug:\n"
"     COMMENT: Debug option\n"
"     DESCRIPTION: |\n"
"       The debug mode is enable mainly for memory profiling.\n"
"     default: No\n"
"#  profiling_depth:\n"
"#     COMMENT: Maximum level of the profiling for the tracking of the routines\n"
"#     default: -1\n"
"   fftcache:\n"
"     COMMENT: Cache size for the FFT\n"
"     DESCRIPTION: |\n"
"       Specify the cache size for FFT in kBytes.\n"
"     default: 8192\n"
"   accel:\n"
"     COMMENT: Acceleration (hardware)\n"
"     DESCRIPTION: |\n"
"       Specify the use of CUDA (resp. OCL) versions of various subroutines. For CUDA,\n"
"       a GPU.config file is needed.\n"
"     EXCLUSIVE:\n"
"       NO:      No material acceleration\n"
"       CUDAGPU: CUDA\n"
"       OCLGPU:  OpenCL on GPU\n"
"       OCLCPU:  OpenCL on CPU\n"
"       OCLACC:  ???\n"
"     default:   NO\n"
"   ocl_platform:\n"
"     COMMENT: Chosen OCL platform\n"
"     default: ~\n"
"   ocl_devices:\n"
"     COMMENT: Chosen OCL devices\n"
"     default: ~\n"
"   blas:\n"
"     COMMENT: CUBLAS acceleration\n"
"     DESCRIPTION: |\n"
"        Use or not the CUBlas acceleration.\n"
"     default: No\n"
"   projrad:\n"
"     COMMENT: Radius of the projector as a function of the maxrad\n"
"     RANGE: [0., 100.]\n"
"     default: 15.\n"
"   exctxpar:\n"
"     COMMENT: Exact exchange parallelisation scheme\n"
"     EXCLUSIVE:\n"
"       BC: Blocking Collective\n"
"       OP2P: Overlap Point-to-Point\n"
"     default: OP2P\n"
"   ig_diag:\n"
"     COMMENT: Input guess (T=Direct, F=Iterative) diag. of Ham.\n"
"     default: Yes\n"
"   ig_norbp:\n"
"     COMMENT: Input guess Orbitals per process for iterative diag.\n"
"     RANGE: [1, 1000]\n"
"     default: 5\n"
"   ig_blocks:\n"
"     COMMENT: Input guess Block sizes for orthonormalisation\n"
"     RANGE: [1, 100000]\n"
"     default: [300, 800]\n"
"   ig_tol:\n"
"     COMMENT: Input guess Tolerance criterion\n"
"     RANGE: [0., 1.]\n"
"     default: 1.0e-4\n"
"   methortho:\n"
"     COMMENT: Orthogonalisation\n"
"     EXCLUSIVE:\n"
"       0: Cholesky\n"
"       1: Gram-Schmidt/Cholesky\n"
"       2: Loewdin\n"
"     default: 0\n"
"   rho_commun:\n"
"     COMMENT: Density communication scheme (DBL, RSC, MIX)\n"
"     default: DEF\n"
"   unblock_comms:\n"
"     COMMENT: Overlap Communications of fields (OFF,DEN,POT)\n"
"     EXCLUSIVE:\n"
"       OFF: Synchronous run\n"
"       DEN: ???\n"
"       POT: ???\n"
"     default: OFF\n"
"   linear:\n"
"     COMMENT: Linear Input Guess approach\n"
"     EXCLUSIVE:\n"
"       OFF: ???\n"
"       LIG: ???\n"
"       FUL: ???\n"
"       TMO: ???\n"
"     default: OFF\n"
"   tolsym:\n"
"     COMMENT: Tolerance for symmetry detection\n"
"     RANGE: [0., 1.]\n"
"     default: 1.0e-8\n"
"   signaling:\n"
"     COMMENT: Expose calculation results on Network\n"
"     default: No\n"
"   signaltimeout:\n"
"     COMMENT: Time out on startup for signal connection (in seconds)\n"
"     RANGE: [-1, 3600]\n"
"     default: 0\n"
"   domain:\n"
"     COMMENT: Domain to add to the hostname to find the IP\n"
"     default: ~\n"
"   inguess_geopt:\n"
"     COMMENT: Input guess to be used during the optimization\n"
"     EXCLUSIVE:\n"
"       0: Wavelet input guess\n"
"       1: Real space input guess\n"
"     default: 0\n"
"   store_index:\n"
"     COMMENT: Store indices or recalculate them for linear scaling\n"
"     default: Yes\n"
"   psp_onfly:\n"
"     COMMENT: Calculate pseudopotential projectors on the fly\n"
"     default: Yes\n"
"   multipole_preserving:\n"
"     COMMENT: (EXPERIMENTAL) Preserve the multipole moments of the ionic density\n"
"     default: No\n"
"   mp_isf:\n"
"     COMMENT: (EXPERIMENTAL) Interpolating scaling function or lifted dual order for the multipole preserving\n"
"     default: 16\n"
"   mixing_after_inputguess:\n"
"     COMMENT: Mixing step after linear input guess\n"
"     EXCLUSIVE:\n"
"       0: keep the initial input guess density\n"
"       1: perform a mixing step with the default mixing parameter\n"
"       2: keep the new density, completely neglecting the input guess density\n"
"     default: 1\n"
"   iterative_orthogonalization:\n"
"     COMMENT: Iterative_orthogonalization for input guess orbitals\n"
"     default: No\n"
"   check_sumrho:\n"
"     COMMENT: Enables linear sumrho check\n"
"     default: 1\n"
"     EXCLUSIVE:\n"
"       0: No check\n"
"       1: Light check\n"
"       2: Full check\n"
"   check_overlap:\n"
"     COMMENT: Enables linear overlap check\n"
"     default: 1\n"
"     EXCLUSIVE:\n"
"       0: No check\n"
"       1: Light check\n"
"       2: Full check\n"
"   experimental_mode:\n"
"     COMMENT: Activate the experimental mode in linear scaling\n"
"     default: No\n"
"   explicit_locregcenters:\n"
"     COMMENT: Linear scaling explicitly specify localization centers\n"
"     default: No\n"
"   calculate_KS_residue:\n"
"     COMMENT: Linear scaling calculate Kohn-Sham residue\n"
"     default: Yes\n"
"   intermediate_forces:\n"
"     COMMENT: Linear scaling calculate intermediate forces\n"
"     default: No\n"
"   kappa_conv:\n"
"     COMMENT: Exit kappa for extended input guess (experimental mode)\n"
"     default: 0.1\n"
"   calculate_gap:\n"
"     COMMENT: Linear scaling calculate the HOMO LUMO gap\n"
"     default: No\n"
"   loewdin_charge_analysis:\n"
"     COMMENT: Linear scaling perform a Loewdin charge analysis at the end of the calculation\n"
"     default: No\n"
"   coeff_weight_analysis:\n"
"     COMMENT: Linear scaling perform a Loewdin charge analysis of the coefficients for fragment calculations\n"
"     default: No\n"
"   check_matrix_compression:\n"
"     COMMENT: Linear scaling perform a check of the matrix compression routines\n"
"     default: Yes\n"
"   correction_co_contra:\n"
"     COMMENT: Linear scaling correction covariant / contravariant gradient\n"
"     default: Yes\n"
"   FOE_restart:\n"
"     COMMENT: Restart method to be used for the FOE method\n"
"     EXCLUSVE:\n"
"       0: AO\n"
"       1: REFORMAT\n"
"     default: 0\n"
"     ao: 0\n"
"     reformat: 1\n"
"   imethod_overlap:\n"
"     COMMENT: Method to calculate the overlap matrices (1=old, 2=new)\n"
"     EXCLUSIVE:\n"
"       1: old\n"
"       2: new\n"
"     default: 1\n"
"   enable_matrix_taskgroups:\n"
"     COMMENT: Enable the matrix taskgroups\n"
"     default: True\n"
"   hamapp_radius_incr:\n"
"     COMMENT: Radius enlargement for the Hamiltonian application (in grid points)\n"
"     default: 8\n"
"     RANGE: [0,8]\n"
"   adjust_kernel_iterations:\n"
"     COMMENT: Enable the adaptive ajustment of the number of kernel iterations\n"
"     default: True\n"
"   adjust_kernel_threshold:\n"
"     COMMENT: Enable the adaptive ajustment of the kernel convergence threshold according to the support function convergence\n"
"     default: True\n"
"   wf_extent_analysis:\n"
"     COMMENT: Perform an analysis of the extent of the support functions (and possibly KS orbitals)\n"
"     default: False\n"
"   foe_gap:\n"
"     COMMENT: Use the FOE method to calculate the HOMO-LUMO gap at the end of a calculation\n"
"     default: False\n"
" lin_general:\n"
"     hybrid:\n"
"       COMMENT: Activate the hybrid mode; if activated, only the low accuracy values will be relevant\n"
"       default: No\n"
"     nit:\n"
"       COMMENT: Number of iteration with low/high accuracy\n"
"       DESCRIPTION: |\n"
"         Maximal number of outer loop iterations (high / low accuracy).\n"
"         In each iteration both the support function and the kernel are optimized.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [0,1000]\n"
"       PROFILE_FROM: hybrid\n"
"       default: [100,100]\n"
"       Yes: [100, 1]\n"
"     rpnrm_cv:\n"
"       COMMENT: Convergence criterion for low/high accuracy\n"
"       DESCRIPTION: |\n"
"         Convergence criterion for the outer loop iterations (high / low accuracy).\n"
"         The optimization stops if the mean change of the charge density / potential falls below this threshold.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [1.e-15,1.e-5]\n"
"       default: [1.e-12,1.e-12]\n"
"     conf_damping:\n"
"       COMMENT: How the confinement should be decreased, only relevant for hybrid mode; negative -> automatic\n"
"       DESCRIPTION: |\n"
"         Prescription how the prefactor for the confining potential should be decreased\n"
"         * negative value => automatic adjustment (recommended)\n"
"         * positive value => at each iteration the prefactor is multiplied with the specified value\n"
"         This variable is only relevant for the hybrid mode.\n"
"       RANGE: [-1.0,1.0]\n"
"       default: -0.5\n"
"     taylor_order:\n"
"       COMMENT: Order of the Taylor approximation; 0 -> exact\n"
"       DESCRIPTION: |\n"
"         Indicates how the matrix powers are calculated:\n"
"         * n<0 => Laura's method of order n\n"
"         * n=0 => exact calculation\n"
"         * n>0 => Taylor series of order n\n"
"       RANGE: [-100,10000]\n"
"       default: 0\n"
"     max_inversion_error:\n"
"       COMMENT: Linear scaling maximal error of the Taylor approximations to calculate the inverse of the overlap matrix\n"
"       DESCRIPTION: |\n"
"         Maximal error of the Taylor approximations which are used to  calculate the inverse of the overlap matrix.\n"
"         If this error is larger than the specified value, the order of the Taylor approximation is increased.\n"
"         The default value is set very large, such such in this case no automatic adjustment is performed.\n"
"       default: 1.d0\n"
"       RANGE: [1.e-20,1.0]\n"
"     output_wf:\n"
"       COMMENT: Output basis functions; 0 no output, 1 formatted output, 2 Fortran bin, 3 ETSF\n"
"       DESCRIPTION: |\n"
"         Indicates how the support functions should be written to disk at the end of the run:\n"
"          *  0 => None\n"
"          *  1 => Wavelet, Plain text\n"
"          *  2 => Wavelet, Fortran binary\n"
"          *  3 => Wavelet, ETSF file format\n"
"          * 10 => None\n"
"          * 11 => Wavelet and Interpolating Scaling Function (density), Plain text\n"
"          * 12 => Wavelet and Interpolating Scaling Function (density), Fortran binary\n"
"          * 13 => Wavelet and Interpolating Scaling Function (density), ETSF file format\n"
"          * 20 => None\n"
"          * 21 => Wavelet and Interpolating Scaling Function (wavefunction), Plain text\n"
"          * 22 => Wavelet and Interpolating Scaling Function (wavefunction), Fortran binary\n"
"          * 23 => Wavelet and Interpolating Scaling Function (wavefunction), ETSF file format\n"
"       default: 0\n"
"       EXCLUSIVE:\n"
"         0:  None\n"
"         1:  Wavelet, Plain text\n"
"         2:  Wavelet, Fortran binary\n"
"         3:  Wavelet, ETSF file format\n"
"         10: None\n"
"         11: Wavelet and Interpolating Scaling Function, Plain text\n"
"         12: Wavelet and Interpolating Scaling Function, Fortran binary\n"
"         13: Wavelet and Interpolating Scaling Function, ETSF file format\n"
"         10: None\n"
"         11: Wavelet and Interpolating Scaling Function (density), Plain text\n"
"         12: Wavelet and Interpolating Scaling Function (density), Fortran binary\n"
"         13: Wavelet and Interpolating Scaling Function (density), ETSF file format\n"
"         20: None\n"
"         21: Wavelet and Interpolating Scaling Function (wavefunction), Plain text\n"
"         22: Wavelet and Interpolating Scaling Function (wavefunction), Fortran binary\n"
"         23: Wavelet and Interpolating Scaling Function (wavefunction), ETSF file format\n"
"     output_mat:\n"
"       COMMENT: Output sparse matrices; 0 no output, 1 formatted sparse, 11 formatted dense, 21 formatted both\n"
"       DESCRIPTION: |\n"
"         Indicates how the support functions should be written to disk at the end of the run:\n"
"          *  0 => None\n"
"          *  1 => Plain text in sparse format\n"
"          * 11 => Plain text in dense format\n"
"          * 21 => Plain text in both formats\n"
"          *  2 => Fortran binary in sparse format\n"
"          * 12 => Fortran binary in dense format\n"
"          * 22 => Fortran binary in both formats\n"
"          *  4 => MPI native binary in sparse format\n"
"       default: 0\n"
"       EXCLUSIVE:\n"
"         0:  None\n"
"         1:  Plain text sparse\n"
"         11: Plain text dense\n"
"         21: Plain text sparse and dense\n"
"         2:  Fortran binary sparse\n"
"         12: Fortran binary dense\n"
"         22: Fortran binary sparse and dense\n"
"         4:  MPI native binary sparse\n"
"     output_coeff:\n"
"       COMMENT: Output KS coefficients; 0 no output, 1 formatted output\n"
"       DESCRIPTION: |\n"
"         Indicates how the support functions should be written to disk at the end of the run:\n"
"          *  0 => None\n"
"          *  1 => Plain text\n"
"       default: 0\n"
"       EXCLUSIVE:\n"
"         0:  None\n"
"         1:  Plain text\n"
"     output_fragments:\n"
"       COMMENT: Output support functions, kernel and coeffs; 0 fragments and full system, 1 fragments only, 2 full system only\n"
"       DESCRIPTION: |\n"
"         Indicates how the support functions should be written to disk at the end of the run:\n"
"          *  0 => Fragments and full system\n"
"          *  1 => Fragments only\n"
"          *  2 => Full system only\n"
"       default: 0\n"
"       EXCLUSIVE:\n"
"         0:  Fragments and full system\n"
"         1:  Fragments only\n"
"         2:  Full system only\n"
"     kernel_restart_mode:\n"
"       COMMENT: Method for restarting kernel; 0 kernel, 1 coefficients, 2 random, 3 diagonal, 4 support function weights\n"
"       DESCRIPTION: |\n"
"         Indicates how the kernel should be generated for a restart calculation:\n"
"          *  0 => Density kernel\n"
"          *  1 => Coefficients\n"
"          *  2 => Random\n"
"          *  3 => Diagonal kernel, i.e. support functions scaled by number of electrons (per fragment)\n"
"          *  4 => Support functions weights\n"
"       default: 0\n"
"       EXCLUSIVE:\n"
"         0:  Density kernel\n"
"         1:  Coefficients\n"
"         2:  Random\n"
"         3:  Diagonal kernel\n"
"         4:  Support function weights\n"
"       #Here follow some shortcuts\n"
"       kernel: 0\n"
"       coeff: 1\n"
"       random: 2\n"
"       diag: 3\n"
"       weight: 4\n"
"     kernel_restart_noise:\n"
"       COMMENT: Add random noise to kernel or coefficients when restarting\n"
"       DESCRIPTION: |\n"
"         Indicates how much noise should be added to kernel or coefficients for a restart calculation\n"
"       default: 0.0d0\n"
"       RANGE: [0.0,1.0]\n"
"     frag_num_neighbours:\n"
"       COMMENT: Number of neighbours to output for each fragment\n"
"       DESCRIPTION: |\n"
"         Indicates how many neighbouring atoms should be written to frag_env.xyz  - only allow for a small number of neighbours\n"
"       RANGE: [0,4]\n"
"       default: 0\n"
"     frag_neighbour_cutoff:\n"
"       COMMENT: Number of neighbours to output for each fragment\n"
"       DESCRIPTION: |\n"
"         Distance (in bohr) beyond which neighbouring atoms are not included in frag_env.xyz\n"
"       RANGE: [0,100]\n"
"       default: 12.0d0\n"
"     cdft_lag_mult_init:\n"
"       COMMENT: CDFT initial value for Lagrange multiplier\n"
"       DESCRIPTION: |\n"
"         The sign is set automatically based on the charge to be constrained\n"
"       default: 0.05d0\n"
"       RANGE: [0.0,10.0]\n"
"     cdft_conv_crit:\n"
"       COMMENT: CDFT convergence threshold for the constrained charge\n"
"       DESCRIPTION: |\n"
"         possible value: 1 positive real number\n"
"       default: 1.e-2\n"
"       RANGE: [1.e-20,1000.0]\n"
"     cdft_nit:\n"
"       COMMENT: Number of iterations for CDFT loop over V_c\n"
"       default: 100\n"
"       RANGE: [1,1000]\n"
"     calc_dipole:\n"
"       DESCRIPTION: |\n"
"         Indicates whether the dipole moment should be calculated after the run or not\n"
"       COMMENT: Calculate dipole\n"
"       default: No\n"
"     calc_quadrupole:\n"
"       DESCRIPTION: |\n"
"         Indicates whether the quadrupole moment should be calculated after the run or not\n"
"       COMMENT: Calculate quadrupole\n"
"       default: No\n"
"# this name might be misleading\n"
"     subspace_diag: #called calc_ks, to be renamed\n"
"       COMMENT: Diagonalization at the end\n"
"       DESCRIPTION: |\n"
"         Indicates whether a diagonalization shall be performed at the end of the run\n"
"       default: No\n"
"     extra_states:\n"
"       COMMENT: Number of extra states to include in support function and kernel optimization (dmin only), must be equal to norbsempty\n"
"       default: 0\n"
"       RANGE: [0, 10000]\n"
"     calculate_onsite_overlap:\n"
"       COMMENT: Calculate the onsite overlap matrix (has only an effect if the matrices are all written to disk)\n"
"       default: No\n"
"     charge_multipoles:\n"
"       COMMENT: Calculate the atom-centered multipole coefficients; 0 no, 1 old approach Loewdin, 2 new approach Projector\n"
"       DESCRIPTION: |\n"
"         Calculate the multipole expansion coefficients of the atomic charge which can then be used to establish a QM/MM coupling.\n"
"         Currently only implemented and tested for nspin=1\n"
"         Possible values are:\n"
"         * 0 => no calculation\n"
"                    * 1 => Loewdin approach, however without explicit orthogonalization\n"
"                    #* 2 => new approach using simple projector and orthogonalized support functions\n"
"                    #* 3 => new approach using full projector and orthogonalized support functions\n"
"                    * 11 => Mulliken approach\n"
"                    #* 12 => same as 2, but with non-orthogonalized support functions\n"
"                    #* 13 => same as 3, but with non-orthogonalized support functions\n"
"       default: 0\n"
"       EXCLUSIVE:\n"
"         0: no calculation\n"
"         1: Loewdin approach, however without explicit orthogonalization\n"
"         #2: new approach using simple projector and orthogonalized support functions\n"
"         #3: new approach using full projector and orthogonalized support functions\n"
"         11: Mulliken approach\n"
"         #12: same as 2, but with non-orthogonalized support functions\n"
"         #13: same as 3, but with non-orthogonalized support functions\n"
"     support_function_multipoles:\n"
"       COMMENT: Calculate the multipole moments of the support functions\n"
"       default: False\n"
"     plot_locreg_grids:\n"
"       COMMENT: Plot the scaling function and wavelets grid of each localization region\n"
"       default: False\n"
"     calculate_FOE_eigenvalues:\n"
"       COMMENT: First and last eigenvalue to be calculated using the FOE procedure\n"
"       default: [0,-1]\n"
"     precision_FOE_eigenvalues:\n"
"       COMMENT: Decay length of the error function used to extract the eigenvalues (i.e. something like the resolution)\n"
"       default: 5.e-3\n"
"       RANGE: [0.0,1.e-1]\n"
"     #centers_auto:\n"
"     #  default: yes\n"
"     multipole_centers:\n"
"       COMMENT: Determines whether the multipole centers shall be determined automatically (i.e. taking the atoms) or whether they are provided manually\n"
"       #default: yes\n"
"       default: 0.0\n"
"     consider_entropy:\n"
"       COMMENT: Indicate whether the entropy contribution to the total energy shall be considered\n"
"       default: False\n"
"#Parameters for the localised basis set (or support function) for the linear scaling version\n"
" lin_basis:\n"
"     nit:\n"
"       COMMENT: Maximal number of iterations in the optimization of the support functions\n"
"       DESCRIPTION: |\n"
"         Maximal number of iterations in the optimization of the support functions.\n"
"         The first number is used for the low accuracy part, the second one for the high accuracy part.\n"
"         If the hybrid mode is used, only the first value is taken into account.\n"
"       RANGE: [0,100]\n"
"       default: [4,5]\n"
"     nit_ig:\n"
"       COMMENT:  maximal number of iterations to optimize the support functions in the extended input guess (experimental mode only)\n"
"       RANGE: [0,100]\n"
"       default: 50 #just for backward compatibility, should be smaller\n"
"     extended_ig:\n"
"       COMMENT:  whether or not to do an extended input guess (will be activated by default in experimental mode)\n"
"       default: No\n"
"     orthogonalize_sfs:\n"
"       COMMENT:  whether or not to explicitly orthogonalize SFs (will be activated by default except in experimental mode, and may eventually be automatically turned off)\n"
"       default: Yes\n"
"     idsx:\n"
"       COMMENT: DIIS history for optimization of the support functions (low/high accuracy); 0 -> SD\n"
"       DESCRIPTION: |\n"
"         Indicates the method with which the support functions are optimized\n"
"          * n=0 => steepest descent\n"
"          * n>0 => DIIS with history length n\n"
"       RANGE: [0,20]\n"
"       default: [6,6]\n"
"     gnrm_cv:\n"
"       COMMENT: Convergence criterion for the optimization of the support functions (low/high accuracy)\n"
"       DESCRIPTION: |\n"
"         Convergence criterion for optimization of the support functions (high / low accuracy).\n"
"         The optimization stops if the mean gradient norm falls below this threshold.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [1.e-6,1.0]\n"
"       default: [1.e-2,1.e-4]\n"
"     gnrm_ig:\n"
"       COMMENT: Convergence criterion for the optimization of the support functions in the extended input guess (experimental mode only)\n"
"       RANGE: [1.e-5,10.0]\n"
"       default: 1.e-3 #just for backward compatibility, should be larger\n"
"     deltae_cv:\n"
"       COMMENT: Total relative energy difference to stop the optimization ('experimental_mode' only)\n"
"       DESCRIPTION: |\n"
"         The optimization of the support functions stops if the total relative gain in the target function\n"
"         during the given cycle of support function optimization falls below this threshold\n"
"       RANGE: [1.e-6,1.e-1]\n"
"       default: 1.e-4\n"
"     gnrm_dyn:\n"
"       COMMENT: Dynamic convergence criterion ('experimental_mode' only)\n"
"       DESCRIPTION: |\n"
"         From this value the effective dynamic convergence criterion is calculated by a division by kappa.\n"
"         Then the optimization of the support functions stops if the mean gradient norm falls\n"
"         below this effective dynamic convergence criterion\n"
"         and is at the same time smaller than the value of min_gnrm_for_dynamic.\n"
"         The specified value should be approximately 10 times smaller than the value of gnrm_cv.\n"
"         This value is only used for the experimental mode.\n"
"       RANGE: [1.e-7,1.e-1]\n"
"       default: 1.e-4\n"
"     min_gnrm_for_dynamic:\n"
"       COMMENT: Minimal gnrm to active the dynamic gnrm criterion\n"
"       DESCRIPTION: |\n"
"         The dynamic convergence criterion (specified by gnrm_dyn) is only active if the mean gradient norm\n"
"         of the support functions is below this threshold\n"
"       RANGE: [1.e-7,1.0]\n"
"       default: 1.e-3\n"
"     alpha_diis:\n"
"       COMMENT: Multiplicator for DIIS\n"
"       DESCRIPTION: |\n"
"                    possible value: 2 positive real numbers\n"
"                    The support functions gradient is multiplied by this number before being used for the DIIS optimization.\n"
"                    The first entry for the low accuracy part, the second one for the high accuracy part.\n"
"                    When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [0.1,10]\n"
"       default: 1.0\n"
"     alpha_sd:\n"
"       COMMENT: Initial step size for SD\n"
"       DESCRIPTION: |\n"
"         The initial step size which is used to optimize the support functions with steepest descent.\n"
"         If the specified value is not appropriate, it will be adjusted automatically.\n"
"         The first entry is used for the low accuracy part, the second one for the high accuracy part.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [1.e-4,10.0]\n"
"       default: 1.0\n"
"     nstep_prec:\n"
"       COMMENT: Number of iterations in the preconditioner\n"
"       DESCRIPTION: |\n"
"         Indicates how many iterations in the preconditioning of the gradient shall be performed\n"
"       RANGE: [1,30]\n"
"       default: 5\n"
"     fix_basis:\n"
"       COMMENT: Fix the support functions if the density change is below this threshold\n"
"       RANGE: [1.e-14,1.e-6]\n"
"       default: 1.e-10\n"
"     correction_orthoconstraint:\n"
"       COMMENT: Correction for the slight non-orthonormality in the orthoconstraint\n"
"       DESCRIPTION: |\n"
"         Indicates whether the slight non-orthogonality of the support functions\n"
"         should be taken into account during the application of the orthogonality constraint\n"
"           - 0 => take into account\n"
"           - 1 => don't take into account\n"
"       default: 1\n"
"       RANGE:  [0,1]\n"
"     orthogonalize_ao:\n"
"       COMMENT: Orthogonalize the atomic orbitals used as input guess\n"
"       default: Yes\n"
"     reset_DIIS_history:\n"
"       COMMENT: Reset the DIIS history when starting the loop which optimizes the support functions\n"
"       default: No\n"
" #Parameters for the calculation of the kernel (linear scaling version)\n"
" lin_kernel:\n"
"     nstep:\n"
"       COMMENT: Number of steps taken when updating the coefficients via direct minimization for each iteration of the density kernel loop\n"
"       DESCRIPTION: |\n"
"         Indicates the number of steps taken when updating the coefficients\n"
"         via direct minimization for each iteration of the density kernel loop.\n"
"         The first entry is used for the low accuracy part, the second one for the high accuracy part.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [0,100]\n"
"       default: [1,1]\n"
"     nit:\n"
"       COMMENT: Number of iterations in the (self-consistent) optimization of the density kernel\n"
"       DESCRIPTION: |\n"
"         Indicates the number of iterations in the (self-consistent) optimization of the density kernel.\n"
"         The first entry is used for the low accuracy part, the second one for the high accuracy part.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [0,1000]\n"
"       default: [5,5]\n"
"     # check whether there are different DIIS histories...\n"
"     idsx_coeff:\n"
"       COMMENT: DIIS history for direct mininimization\n"
"       default: [0,0]\n"
"     idsx:\n"
"       COMMENT: Mixing method; 0 -> linear mixing, >=1 -> Pulay mixing\n"
"       DESCRIPTION: |\n"
"         Indicates how the density / potential mixing shall be performed:\n"
"         n=0 => linear mixing\n"
"         n=1 => Pulay mixing with history length n\n"
"         The first entry is used for the low accuracy part, the second one for the high accuracy part.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [0,20]\n"
"       default: [0,0]\n"
"     alphamix:\n"
"       COMMENT: Mixing parameter (low/high accuracy)\n"
"       DESCRIPTION: |\n"
"         Indicates the mixing parameter used for the mixing of the charge density / potential.\n"
"         The larger the value, the more of the previous value will be kept.\n"
"         The first entry is used for the low accuracy part, the second one for the high accuracy part.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [0.0,1.0]\n"
"       default: [0.5,0.5]\n"
"     gnrm_cv_coeff:\n"
"       COMMENT: Convergence criterion on the gradient for direct minimization\n"
"       DESCRIPTION: |\n"
"         Convergence criterion on the gradient for direct minimization.\n"
"         The first entry is used for the low accuracy part, the second one for the high accuracy part.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       default: [1.e-5,1.e-5]\n"
"     rpnrm_cv:\n"
"       COMMENT: Convergence criterion (change in density/potential) for the kernel optimization\n"
"       DESCRIPTION: |\n"
"         The optimization of the density kernel is stopped as soon as the mean change\n"
"         of the charge density / potential between two iterations falls below this threshold.\n"
"         The first entry is used for the low accuracy part, the second one for the high accuracy part.\n"
"         When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [1.e-14,1.e-5]\n"
"       default: [1.e-10,1.e-10]\n"
"     linear_method:\n"
"       COMMENT: Method to optimize the density kernel\n"
"       DESCRIPTION: |\n"
"         Indicates the method with which the density kernel is optimized:\n"
"         * DIRMIN => direct minimization\n"
"         * DIAG => diagonalization with density mixing\n"
"         * FOE => Fermi Operator Expansion\n"
"         * PEXSI => PEXSI library\n"
"       EXCLUSIVE:\n"
"         DIRMIN: direct minimization\n"
"         DIAG: diagonalization\n"
"         FOE: Fermi Operator Expansion\n"
"         PEXSI: external PEXSI lbrary\n"
"       default: DIAG\n"
"     mixing_method:\n"
"       COMMENT: Quantity to be mixed\n"
"       DESCRIPTION: |\n"
"         Indicates which quantity should be used for the mixing procedure:\n"
"          DEN => density mixing\n"
"          POT => potential mixing\n"
"       EXCLUSIVE:\n"
"         DEN: density mixing\n"
"         POT: potential mixing\n"
"       default: DEN\n"
"     alpha_sd_coeff:\n"
"       COMMENT: Initial step size for SD for direct minimization\n"
"       default: 0.2\n"
"     alpha_fit_coeff:\n"
"       COMMENT: Update the SD step size by fitting a quadratic polynomial\n"
"       DESCRIPTION: |\n"
"         Update the SD step size by fitting a quadratic polynomial to the energy\n"
"         along the direction of the gradient during direct mininimization.\n"
"       default: No\n"
"     coeff_scaling_factor:\n"
"       COMMENT: Factor to scale the gradient in direct minimization\n"
"       DESCRIPTION: |\n"
"         Specifies a scaling factor for the coefficient gradient, recommended to be around 0.1 - 0.5 for CDFT, fragment calculations\n"
"         or systems where convergence is unstable.\n"
"       default: 1.0\n"
"       RANGE: [0.0,1.e1]\n"
"     delta_pnrm:\n"
"       COMMENT: Stop the kernel optimization if the density/potential difference has decreased by this factor (deactivated if -1.0)\n"
"       DESCRIPTION: |\n"
"         stop the kernel optimization if the density/potential difference has decreased by this factor compared to the first iteration.\n"
"         The feature is deactivated for a value of -1.0\n"
"       default: -1.0\n"
"       RANGE: [-1.0,1.0]\n"
"#Parameters to define the localized basis set or support functions (linear scaling version)\n"
" lin_basis_params:\n"
"     nbasis:\n"
"       COMMENT: Number of support functions per atom\n"
"       DESCRIPTION: |\n"
"                    Indicates the number of support fuctions centered on this atom type. Possible values are:\n"
"                     * 1 (s orbital type)\n"
"                     * 4 (s,p orbital types)\n"
"                     * 9 (s,p,d orbital types)\n"
"                     * 16 (s,p,d,f orbital types)\n"
"       default: 4\n"
"     ao_confinement:\n"
"       COMMENT: Prefactor for the input guess confining potential\n"
"       DESCRIPTION: |\n"
"                    Indicates the prefactor of the confining potential\n"
"                    of this atom type which is used to generate the input guess.\n"
"       RANGE: [-1.0,10.0]\n"
"       default: 8.3e-3\n"
"     confinement:\n"
"       COMMENT: Prefactor for the confining potential (low/high accuracy)\n"
"       DESCRIPTION: |\n"
"                    Indicates the prefactor of the confining potential of this atom type.\n"
"                    The first entry is used for the low accuracy part, the second one for the high accuracy part.\n"
"                    When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [-1.0,10.0]\n"
"       default: [8.3e-3,0.0]\n"
"     rloc:\n"
"       COMMENT: Localization radius for the support functions\n"
"       DESCRIPTION: |\n"
"                    Indicates the cutoff radius for the support functions.\n"
"                    The first entry is used for the low accuracy part, the second one\n"
"                    for the high accuracy part; the second value must not be smaller than the first one.\n"
"                    When the hybrid mode is used, only the first entry is taken into account.\n"
"       RANGE: [2.0,1000.0]\n"
"       default: [7.0,7.0]\n"
"     rloc_kernel:\n"
"       COMMENT: Localization radius for the density kernel\n"
"       DESCRIPTION: |\n"
"                    Indicates the cutoff radius for the density kernel.\n"
"                    Must be at least as large as the support function cutoff\n"
"                    plus 8 times the grid spacing; will be adjusted automatically if it is chosen too small.\n"
"       RANGE: [2.0,1000.0]\n"
"       default: 9.0\n"
"     rloc_kernel_foe:\n"
"       COMMENT: Cutoff radius for the FOE matrix vector multiplications\n"
"       DESCRIPTION: |\n"
"                    Indicates the cutoff radius used for the matrix vector multiplications.\n"
"                    Must be at least as large as the kernel cutoff.\n"
"       RANGE: [2.0,1000]\n"
"       default: 14.0\n"
" ## chess:\n"
" ##   ef_interpol_det:\n"
" ##     COMMENT: FOE max determinant of cubic interpolation matrix\n"
" ##     RANGE: [0., 1.]\n"
" ##     default: 1.e-12\n"
" ##   ef_interpol_chargediff:\n"
" ##     COMMENT: FOE max charge difference for interpolation\n"
" ##     RANGE: [0., 1000.]\n"
" ##     default: 1.0\n"
" ##   blocksize_pdsyev:\n"
" ##     COMMENT: SCALAPACK linear scaling blocksize\n"
" ##     default: -8\n"
" ##   pdgemm_blocksize:\n"
" ##     COMMENT: SCALAPACK linear scaling blocksize\n"
" ##     default: -8\n"
" ##   maxproc_pdsyev:\n"
" ##     COMMENT: SCALAPACK linear scaling max num procs\n"
" ##     default: 4\n"
" ##   maxproc_pdgemm:\n"
" ##     COMMENT: SCALAPACK linear scaling max num procs\n"
" ##     default: 4\n"
" ##   evbounds_nsatur:\n"
" ##     COMMENT: Number of FOE cycles before the eigenvalue bounds are shrinked (linear)\n"
" ##     default: 3\n"
" ##   evboundsshrink_nsatur:\n"
" ##     COMMENT: Maximal number of unsuccessful eigenvalue bounds shrinkings\n"
" ##     default: 4\n"
" ##   fscale_lowerbound:\n"
" ##     COMMENT: Linear scaling lower bound for the error function decay length\n"
" ##     RANGE: [1.e-4,1.e-2]\n"
" ##     default: 5.e-3\n"
" ##   fscale_upperbound:\n"
" ##     COMMENT: Linear scaling upper bound for the error function decay length\n"
" ##     default: 5.e-2\n"
" ##     RANGE: [1.e-2,1.0]\n"
" ##   eval_range_foe:\n"
" ##     COMMENT: Lower and upper bound of the eigenvalue spectrum, will be adjusted automatically if chosen unproperly\n"
" ##     DESCRIPTION: |\n"
" ##       Indicates the lower and upper bound of the eigenvalue spectrum which is used for the Fermi Operator Expansion.\n"
" ##       This spectrum will be adjusted automatically if not chosen appropriately.\n"
" ##     RANGE: [-10.0,10.0]\n"
" ##     default: [-0.5,0.5]\n"
" ##   fscale:\n"
" ##     COMMENT: Decay length of the error function\n"
" ##     DESCRIPTION: |\n"
" ##       Indicates the decay length of the error function which is used by the Fermi Operator Expasion to construct the density kernel.\n"
" ##       This value will be adjusted automatically if not chosen appropriately.\n"
" ##     RANGE: [1.e-3,5.e-2]\n"
" ##     default: 2.e-2\n"
" ##   pexsi_npoles:\n"
" ##     COMMENT: Number of poles used by PEXSI\n"
" ##     DESCRIPTION: number of poles used by PEXSI to calculate the density kernel from the Hamiltonian and overlap matrices\n"
" ##     default: 40\n"
" ##     RANGE: [20,200]\n"
" ##   pexsi_mumin:\n"
" ##     COMMENT: Initial guess for the lower bound of the chemical potential used by PEXSI\n"
" ##     DESCRIPTION: |\n"
" ##       Initial guesss for the lower bound of the chemical potential (in hartree)\n"
" ##       used by PEXSI, will be adjusted automatically later.\n"
" ##     default: -1.0\n"
" ##     RANGE: [-5.0,5.0]\n"
" ##   pexsi_mumax:\n"
" ##     COMMENT: Initial guess for the upper bound of the chemical potential used by PEXSI\n"
" ##     DESCRIPTION: initial guesss for the upper bound of the chemical potential (in hartree) used by PEXSI, will be adjusted automatically later\n"
" ##     default: 1.0\n"
" ##     RANGE: [-5.0,5.0]\n"
" ##   pexsi_mu:\n"
" ##     COMMENT: Initial guess for the  chemical potential used by PEXSI\n"
" ##     DESCRIPTION: initial guesss for the chemical potential (in hartree) used by PEXSI, will be adjusted automatically later\n"
" ##     default: 0.5\n"
" ##     RANGE: [-5.0,5.0]\n"
" ##   pexsi_temperature:\n"
" ##     COMMENT: Temperature used by PEXSI\n"
" ##     DESCRIPTION: temperature (in atomic units) used by PEXSI\n"
" ##     default: 1.e-3\n"
" ##     RANGE: [0.0,1.0]\n"
" ##   pexsi_tol_charge:\n"
" ##     COMMENT: Charge tolerance used PEXSI\n"
" ##     DESCRIPTION: tolerance on the number of electrons used by PEXSI\n"
" ##     default: 1.e-3\n"
" ##     RANGE: [1.e-10,1.0]\n"
" ##   pexsi_np_sym_fact:\n"
" ##     COMMENT: Number of tasks for the symbolic factorization used by PEXSI\n"
" ##     DESCRIPTION: Number of tasks used for the symbolic factorization within PEXSI. Too large values can apparently decrease the performance\n"
" ##     default: 16\n"
" ##     RANGE: [1,10000]\n"
"---\n"
"#here starts the document of possible profiles, that have to be evoked with the 'import' keyword\n"
"linear_accurate:\n"
"  COMMENT: Appropriate input parameters for a high accuracy linear scaling calculation\n"
"  DESCRIPTION: |\n"
"          This profile should be chosen if highly accurate energy differences are required.\n"
"  dft:\n"
"    hgrids: 0.3\n"
"    ixc: PBE\n"
"    inputpsiid: linear\n"
"  chess:\n"
"    foe:\n"
"      ef_interpol_chargediff: 1.0\n"
"  perf:\n"
"    experimental_mode: Yes\n"
"    correction_co_contra: Yes\n"
"    #check_sumrho: 1\n"
"    #calculate_KS_residue: No\n"
"    #check_matrix_compression: No\n"
"  lin_general:\n"
"    hybrid: Yes\n"
"    taylor_order: 1015\n"
"    max_inversion_error: 5.0e-8\n"
"  lin_basis:\n"
"    nit: 8\n"
"    idsx: 8\n"
"    gnrm_cv: 1.e-3\n"
"    gnrm_dyn: 1.e-4\n"
"    deltae_cv: 5.e-5\n"
"    min_gnrm_for_dynamic: 2.e-3\n"
"    alpha_diis: 0.5\n"
"    alpha_sd: 0.5\n"
"    nstep_prec: 6\n"
"    fix_basis: 1.0e-12\n"
"    correction_orthoconstraint: 0\n"
"  lin_kernel:\n"
"    nit: 4\n"
"    idsx: 6\n"
"    alphamix: 0.1\n"
"    rpnrm_cv: [1.e-12,1.e-12]\n"
"    linear_method: FOE\n"
"  lin_basis_params:\n"
"    ao_confinement: -1.0\n"
"    confinement: -1.0\n"
"    rloc_kernel: 10.0\n"
"    rloc_kernel_foe: 12.0\n"
"    H:  {nbasis: 1, rloc: 5.0}\n"
"    C:  {nbasis: 4, rloc: 5.5}\n"
"    N:  {nbasis: 4, rloc: 5.5}\n"
"    O:  {nbasis: 4, rloc: 5.5}\n"
"    P:  {nbasis: 4, rloc: 6.0}\n"
"    S:  {nbasis: 4, rloc: 6.0}\n"
"    Cl: {nbasis: 4, rloc: 6.0}\n"
"    Na: {nbasis: 1, rloc: 7.5}\n"
"    K:  {nbasis: 1, rloc: 8.0}\n"
"    Sn: {nbasis: 4, rloc: 6.5}\n"
"\n"
"\n"
"linear_moderate:\n"
"  COMMENT: Appropriate input parameters for a moderate accuracy linear scaling calculation\n"
"  DESCRIPTION: |\n"
"          This profile should be chosen if accurate forces are required,\n"
"          but there is no need for extremely accurate energies.\n"
"  dft:\n"
"    hgrids: 0.3\n"
"    ixc: PBE\n"
"    inputpsiid: linear\n"
"  chess:\n"
"    foe:\n"
"      ef_interpol_chargediff: 1.0\n"
"  perf:\n"
"    experimental_mode: Yes\n"
"    correction_co_contra: Yes\n"
"  lin_general:\n"
"    hybrid: Yes\n"
"    taylor_order: 10\n"
"  lin_basis:\n"
"    nit: [10,10]\n"
"    gnrm_cv: [2.e-3,2.e-3]\n"
"    gnrm_dyn: 2.e-4\n"
"    deltae_cv: 5.e-5\n"
"    min_gnrm_for_dynamic: 4.e-3\n"
"    alpha_diis: 0.5\n"
"    alpha_sd: 0.5\n"
"    nstep_prec: 6\n"
"    fix_basis: 1.e-11\n"
"    correction_orthoconstraint: 0\n"
"  lin_kernel:\n"
"    nit: [3,3]\n"
"    idsx: [6,6]\n"
"    alphamix: [0.2,0.2]\n"
"    rpnrm_cv: [1.e-11,1.e-11]\n"
"    linear_method: FOE\n"
"  lin_basis_params:\n"
"    ao_confinement: -1.0\n"
"    confinement: -1.0\n"
"    rloc_kernel: 10.0\n"
"    rloc_kernel_foe: 12.0\n"
"    H:  {nbasis: 1, rloc: 5.0}\n"
"    C:  {nbasis: 4, rloc: 5.5}\n"
"    N:  {nbasis: 4, rloc: 5.5}\n"
"    O:  {nbasis: 4, rloc: 5.5}\n"
"    P:  {nbasis: 4, rloc: 6.0}\n"
"    S:  {nbasis: 4, rloc: 6.0}\n"
"    Cl: {nbasis: 4, rloc: 6.0}\n"
"    Na: {nbasis: 1, rloc: 7.5}\n"
"    K:  {nbasis: 1, rloc: 8.0}\n"
"    Sn: {nbasis: 4, rloc: 6.5}\n"
"\n"
"\n"
"linear_fast:\n"
"  COMMENT: Appropriate input parameters for a low accuracy linear scaling calculation\n"
"  DESCRIPTION: |\n"
"          This profile should be chosen if speed is more important than accuracy.\n"
"  dft:\n"
"    hgrids: 0.3\n"
"    ixc: PBE\n"
"    inputpsiid: linear\n"
"  perf:\n"
"    ef_interpol_chargediff: 1.0\n"
"    experimental_mode: Yes\n"
"    correction_co_contra: Yes\n"
"  lin_general:\n"
"    hybrid: Yes\n"
"    taylor_order: 10\n"
"  lin_basis:\n"
"    nit: [10,10]\n"
"    gnrm_cv: [4.e-3,4.e-3]\n"
"    gnrm_dyn: 4.e-4\n"
"    deltae_cv: 5.e-5\n"
"    min_gnrm_for_dynamic: 8.e-3\n"
"    alpha_diis: 0.5\n"
"    alpha_sd: 0.5\n"
"    nstep_prec: 6\n"
"    fix_basis: 1.e-10\n"
"    correction_orthoconstraint: 0\n"
"  lin_kernel:\n"
"    nit: [10,10]\n"
"    idsx: [6,6]\n"
"    alphamix: [0.2,0.2]\n"
"    rpnrm_cv: [1.e-10,1.e-10]\n"
"    linear_method: FOE\n"
"  lin_basis_params:\n"
"    ao_confinement: -1.0\n"
"    confinement: -1.0\n"
"    rloc_kernel: 10.0\n"
"    rloc_kernel_foe: 12.0\n"
"    H:  {nbasis: 1, rloc: 5.0}\n"
"    C:  {nbasis: 4, rloc: 5.5}\n"
"    N:  {nbasis: 4, rloc: 5.5}\n"
"    O:  {nbasis: 4, rloc: 5.5}\n"
"    P:  {nbasis: 4, rloc: 6.0}\n"
"    S:  {nbasis: 4, rloc: 6.0}\n"
"    Cl: {nbasis: 4, rloc: 6.0}\n"
"    Na: {nbasis: 1, rloc: 7.5}\n"
"    K:  {nbasis: 1, rloc: 8.0}\n"
"    Sn: {nbasis: 4, rloc: 6.5}\n"
"\n"
"\n"
"linear_laura:\n"
" COMMENT: temporary profile, to be merged\n"
" DESCRIPTION: |\n"
"           Lauras favourite linear parameters\n"
" dft:\n"
"   inputpsiid: linear\n"
" perf:\n"
"   check_sumrho: 0 #1\n"
"   check_overlap: 0 #1\n"
"   experimental_mode: Yes\n"
"   calculate_KS_residue: No\n"
"   check_matrix_compression: No\n"
"   correction_co_contra: Yes\n"
"   store_index: Yes\n"
"   mixing_after_inputguess: 1\n"
"   kappa_conv: 0.1\n"
"   FOE_restart: 1\n"
"   hamapp_radius_incr: 6\n"
"   loewdin_charge_analysis: No\n"
"   #profiling_depth: 0\n"
" lin_general:\n"
"   hybrid: Yes\n"
"   nit: 40 #3 #0\n"
"   taylor_order: 1010 #20 #1005 #1010\n"
"   max_inversion_error: 1.0e-6\n"
"   rpnrm_cv: 1.0e-10 #5.0e-11 #1.0e-10\n"
"   output_wf: 1 #21 #22\n"
"   output_mat: 11\n"
"   #output_fragments: 1\n"
"   calculate_onsite_overlap: Yes\n"
"   charge_multipoles: 11\n"
"   extra_states: 0\n"
" lin_basis:\n"
"   nit: 3 #4 #6\n"
"   idsx: 3 #4 #6\n"
"   #gnrm_cv: 8.0e-4\n"
"   deltae_cv: 1.0e-4\n"
"   min_gnrm_for_dynamic: 4.0e-3\n"
"   alpha_diis: 0.5\n"
"   alpha_sd: 0.5\n"
"   nstep_prec: 6\n"
"   fix_basis: 1.0e-13\n"
"   correction_orthoconstraint: 0\n"
"   gnrm_ig: 1.e-1\n"
" lin_kernel:\n"
"   #nstep: 10\n"
"   nit: 5\n"
"   #idsx_coeff: [6, 0]\n"
"   idsx: 4\n"
"   alphamix: 0.1 #5 #2 #5\n"
"   linear_method: FOE\n"
"   #eval_range_foe: [-1.0, 1.0]\n"
"   #fscale_foe: 5.0E-002\n"
"   #gnrm_cv_coeff: 1.0E-003\n"
"   rpnrm_cv: 1.0E-010\n"
"   #alphamix: 1.0\n"
" #lin_basis_params:\n"
" #  C:\n"
" #    nbasis: 4\n"
" #    ao_confinement: -1.0\n"
" #    confinement: -1.0\n"
" #    rloc: 7.0 #6.9\n"
" #    rloc_kernel: 10.0 #14.0 #10.0\n"
" #    rloc_kernel_foe: 10.0 #14.0 #10.0\n"
" #  H:\n"
" #    nbasis: 1\n"
" #    ao_confinement: -1.0\n"
" #    confinement: -1.0\n"
" #    rloc: 7.0 #6.9\n"
" #    rloc_kernel: 10.0 #14.0 #10.0\n"
" #    rloc_kernel_foe: 10.0 #14.0 #10.0\n"
"\n"
"\n"
"mixing:\n"
"  COMMENT: To be used for metallic systems\n"
"  DESCRIPTION: |\n"
"           Parameters for the mixing scheme associated to the diagonalization method.\n"
"  dft:\n"
"   gnrm_cv: 1.e-6\n"
"   nrepmax: 1\n"
"   itermax: 3\n"
"   ncong: 5\n"
"   idsx: 0\n"
"  mix:\n"
"   iscf: 17\n"
"   itrpmax: 200\n"
"   rpnrm_cv: 1.E-12\n"
"   norbsempty: 120\n"
"   tel: 0.01\n"
"   alphamix: 0.8\n"
"   alphadiis: 1.d0\n"
"\n"
"\n"
"linear:\n"
" COMMENT: Linear parameters sufficient for PDoS, charge analysis, MD\n"
" DESCRIPTION: |\n"
"         Basic parameters to perform a linear calculation.\n"
" dft:\n"
"   disablesym: Yes\n"
"   inputpsiid: linear\n"
" chess:\n"
"   foe:\n"
"     ef_interpol_chargediff: 1.0\n"
"     eval_range_foe: [-1.0, 1.0]\n"
"     fscale: 5.0e-2\n"
" perf:\n"
"   check_sumrho: 1\n"
"   check_overlap: 1\n"
"   experimental_mode: Yes\n"
"   calculate_KS_residue: No\n"
"   check_matrix_compression: No\n"
"   correction_co_contra: Yes\n"
"   store_index: No\n"
"   mixing_after_inputguess: 0\n"
"   kappa_conv: 0.1\n"
"   FOE_restart: 1\n"
"   hamapp_radius_incr: 6\n"
" lin_general:\n"
"   hybrid: Yes\n"
"   nit: 50\n"
"   taylor_order: 1020\n"
"   max_inversion_error: 5.0e-8\n"
"   rpnrm_cv: 1.0e-11\n"
"   output_mat: 1\n"
"   output_coeff: 0\n"
"   charge_multipoles: 11\n"
" lin_basis:\n"
"   nit: 8\n"
"   idsx: 8 #LG: would'n be better to put, say, 4?\n"
"   gnrm_cv: 2.0e-3\n"
"   deltae_cv: 1.0e-4\n"
"   min_gnrm_for_dynamic: 4.0e-3\n"
"   alpha_diis: 0.5\n"
"   alpha_sd: 0.5\n"
"   nstep_prec: 6\n"
"   fix_basis: 1.0e-12\n"
"   correction_orthoconstraint: 0\n"
"   gnrm_ig: 1.e-1\n"
"   reset_DIIS_history: yes\n"
" lin_kernel:\n"
"   nstep: 10\n"
"   nit: 6\n"
"   idsx_coeff: [6, 0]\n"
"   idsx: 6\n"
"   rpnrm_cv: 1.0e-8\n"
"   alphamix: 0.1\n"
"   linear_method: FOE\n"
"   delta_pnrm: 0.1\n"
" lin_basis_params:\n"
"   ao_confinement: -1.0\n"
"   confinement: -1.0\n"
"   rloc_kernel: 10.0\n"
"   rloc_kernel_foe: 12.0\n"
"   H:  {nbasis: 1, rloc: 5.0}\n"
"   Li: {nbasis: 1, rloc: 5.5}\n"
"   C:  {nbasis: 4, rloc: 5.5}\n"
"   N:  {nbasis: 4, rloc: 5.5}\n"
"   O:  {nbasis: 4, rloc: 5.5}\n"
"   F:  {nbasis: 4, rloc: 5.5}\n"
"   P:  {nbasis: 4, rloc: 6.0}\n"
"   S:  {nbasis: 4, rloc: 6.0}\n"
"   Cl: {nbasis: 4, rloc: 6.0}\n"
"   Na: {nbasis: 1, rloc: 7.5}\n"
"   K:  {nbasis: 1, rloc: 8.0}\n"
"   Sn: {nbasis: 4, rloc: 6.5}\n"
"   Fe: {nbasis: 13, rloc: 8.0}\n"
"   Cu: {nbasis: 9, rloc: 7.0}\n"
"   W:  {nbasis: 9, rloc: 7.5}\n"
" ig_occupation:\n"
"   Fe:\n"
"     3s: 2.0\n"
"     3p: 6.0\n"
"     3d:\n"
"       up: 4.0\n"
"       down: 2.0\n"
"     4s: 2.0\n"
"     4p: 0.0\n"
"   Cu:\n"
"     3d: 10.0\n"
"     4s: 1.0\n"
"     4p: 0.0\n"
"   W:\n"
"     6s: 2.0\n"
"     5d: 4.0\n"
"     6p: 0.0\n"
"\n"
"\n"
"linear_fragments:\n"
"  COMMENT: Input parameters to be added to the linear profile for fragment calculationn (templates ready)\n"
"  #import: linear #to be tested\n"
"  dft:\n"
"    inputpsiid: linear_restart\n"
"  lin_general:\n"
"    hybrid: No\n"
"    nit: 1\n"
"    output_wf: 0\n"
"    output_mat: 0\n"
"  lin_kernel:\n"
"    nit: 100 #here it is better to put a large value\n"
"    rpnrm_cv: [5.0E-08, 1.0E-9]\n"
"  lin_basis: {nit: 1}\n"
"  perf:\n"
"    experimental_mode: No\n"
"    adjust_kernel_threshold: No\n"
"    adjust_kernel_iterations: No\n"
"\n"
"\n"
"md:\n"
"  COMMENT: Molecular dynamics input parameters for H2O molecules\n"
"  geopt:\n"
"    method: AB6MD\n"
"    ionmov: 8\n"
"    dtion: 20.0\n"
"    noseinert: 30.0\n"
  ;
void FC_FUNC_(get_input_variables_definition, GET_INPUT_VARIABLES_DEFINITION)(char* db_ptr,int* db_len)
{
  if (*db_len==0)
    {
    *db_len=strlen(input_variables_definition_arr);
    return;
    }
  else
    {
      memcpy(db_ptr,input_variables_definition_arr, sizeof(char) * (*db_len));
    }
}
